/*************************************************************************/
/*                                                                       */
/*   SNU-RT Benchmark Suite for Worst Case Timing Analysis               */
/*   =====================================================               */
/*                              Collected and Modified by S.-S. Lim      */
/*                                           sslim@archi.snu.ac.kr       */
/*                                         Real-Time Research Group      */
/*                                        Seoul National University      */
/*                                                                       */
/*                                                                       */
/*        < Features > - restrictions for our experimental environment   */
/*                                                                       */
/*          1. Completely structured.                                    */
/*               - There are no unconditional jumps.                     */
/*               - There are no exit from loop bodies.                   */
/*                 (There are no 'break' or 'return' in loop bodies)     */
/*          2. No 'switch' statements.                                   */
/*          3. No 'do..while' statements.                                */
/*          4. Expressions are restricted.                               */
/*               - There are no multiple expressions joined by 'or',     */
/*                'and' operations.                                      */
/*          5. No library calls.                                         */
/*               - All the functions needed are implemented in the       */
/*                 source file.                                          */
/*                                                                       */
/*                                                                       */
/*************************************************************************/
/*                                                                       */
/*  FILE: bs.c                                                           */
/*  SOURCE : Public Domain Code                                          */
/*                                                                       */
/*  DESCRIPTION :                                                        */
/*                                                                       */
/*     Binary search for the array of 15 integer elements.               */
/*                                                                       */
/*  REMARK :                                                             */
/*                                                                       */
/*  EXECUTION TIME :                                                     */
/*                                                                       */
/*                                                                       */
/*************************************************************************/
#ifdef TEST
    #include "../mini_printf.h"
    #include "../posix_c.h"
#else
    #include <stdio.h>
#endif


struct DATA {
  int  key;
  int  value;
}  ;

#ifdef DEBUG
	int cnt1;
#endif

int binary_search(struct DATA data[],int x);

void main()
{

    struct DATA data[10000] = {
        {1, -875751},{2, -976069},{3, -33389},{4, 728892},{5, -13597},{6, -655283},{7, 208387},{8, 247263},{9, 552173},{10, -396626},{11, -365866},{12, 22679},{13, 599562},{14, 258658},{15, 331702},{16, 646517},{17, 270585},{18, -97973},{19, -160599},{20, -531771},
        {21, 370030},{22, 320322},{23, -19909},{24, -380442},{25, -538439},{26, 552231},{27, -272624},{28, -205551},{29, 822320},{30, -674935},{31, -325728},{32, -230939},{33, 366147},{34, -598818},{35, 85041},{36, -215717},{37, 782491},{38, -126120},{39, 112679},{40, 361825},
        {41, 117684},{42, -545846},{43, -850000},{44, -116243},{45, 500357},{46, 793015},{47, -697456},{48, 143280},{49, -467740},{50, 113886},{51, 782025},{52, 307844},{53, 112214},{54, -790276},{55, 164914},{56, 421452},{57, -322233},{58, -423953},{59, 301485},{60, -253803},
        {61, 80815},{62, -459357},{63, -720219},{64, 86480},{65, 615417},{66, -597489},{67, -296547},{68, 375855},{69, -246879},{70, -362970},{71, -760040},{72, 390792},{73, -582061},{74, -311034},{75, 1017471},{76, -36968},{77, 85635},{78, 261005},{79, -436553},{80, 690672},
        {81, 608346},{82, 260453},{83, -218431},{84, -605459},{85, 686553},{86, -167992},{87, -798884},{88, 514964},{89, 119504},{90, -144574},{91, -733561},{92, -655463},{93, 738753},{94, -329066},{95, 458199},{96, 864745},{97, 118910},{98, 441277},{99, -729449},{100, 40402},
        {101, 26194},{102, -374157},{103, -443815},{104, 733388},{105, 35855},{106, -242686},{107, 283380},{108, 251635},{109, 309139},{110, -108569},{111, -414564},{112, -685213},{113, -169282},{114, 303396},{115, 541108},{116, 722822},{117, 249583},{118, -633236},{119, 277411},{120, 406087},
        {121, 782749},{122, -726345},{123, -73652},{124, 181591},{125, 288032},{126, -108865},{127, -496520},{128, -482490},{129, -758755},{130, 192107},{131, 31731},{132, 254912},{133, 483009},{134, 322588},{135, 933395},{136, 616507},{137, 156517},{138, -320725},{139, -414851},{140, 556786},
        {141, -583035},{142, 654839},{143, -523550},{144, 437750},{145, 309315},{146, 99674},{147, -546743},{148, 241783},{149, 666557},{150, -701086},{151, -46416},{152, -206497},{153, 732907},{154, -468344},{155, 772556},{156, -590164},{157, 365711},{158, -421228},{159, -667886},{160, -21436},
        {161, -569100},{162, 4687},{163, 136385},{164, -403909},{165, 510734},{166, 78752},{167, -520316},{168, 622648},{169, 216894},{170, -117284},{171, 720952},{172, 135440},{173, -321262},{174, 373842},{175, -696479},{176, -347924},{177, 27156},{178, 454560},{179, 465204},{180, 379510},
        {181, -484103},{182, -449917},{183, 296443},{184, 349610},{185, -176751},{186, 1034297},{187, 712832},{188, -617343},{189, -61033},{190, -450316},{191, 210779},{192, 681459},{193, 453310},{194, -111615},{195, -635587},{196, -698684},{197, 560707},{198, 766795},{199, -501769},{200, 301235},
        {201, -324407},{202, 70907},{203, 561097},{204, 299852},{205, -18593},{206, 3533},{207, -567456},{208, -246673},{209, 787184},{210, -75339},{211, -342581},{212, -686255},{213, 787879},{214, 121198},{215, 420914},{216, -379830},{217, -39640},{218, -206868},{219, 12614},{220, 424388},
        {221, 425664},{222, -246414},{223, 451515},{224, 318059},{225, -228674},{226, -269315},{227, -87594},{228, -364689},{229, -613908},{230, -147179},{231, -172022},{232, -628861},{233, 281747},{234, 50137},{235, 582198},{236, -253087},{237, -213282},{238, 752445},{239, -90098},{240, 641090},
        {241, -483216},{242, 183969},{243, -103367},{244, -577238},{245, 114103},{246, 425975},{247, -695690},{248, -314325},{249, 954168},{250, 19429},{251, -762783},{252, 706787},{253, 706934},{254, 231387},{255, -51541},{256, -444878},{257, -611605},{258, -961928},{259, 562147},{260, -87635},
        {261, 1038281},{262, -569321},{263, 749751},{264, 757977},{265, -688704},{266, -65302},{267, -659321},{268, 581141},{269, 653989},{270, 586959},{271, -48872},{272, -338812},{273, -463467},{274, 96197},{275, -703617},{276, -311961},{277, 678083},{278, 406103},{279, -279661},{280, -696422},
        {281, 319213},{282, 695054},{283, -301072},{284, -235902},{285, -860793},{286, -59647},{287, -721890},{288, -311307},{289, -606446},{290, 516122},{291, 469205},{292, -148915},{293, -773383},{294, -401908},{295, -102424},{296, -584343},{297, 230985},{298, -82554},{299, -416552},{300, 538730},
        {301, -653803},{302, -776033},{303, -842404},{304, -678089},{305, 335253},{306, 377673},{307, -261772},{308, 649099},{309, -205671},{310, -59345},{311, 387966},{312, 413615},{313, 635486},{314, -791943},{315, 393454},{316, 55119},{317, -916720},{318, -696561},{319, 991019},{320, 328539},
        {321, -497919},{322, 703506},{323, 939890},{324, -191104},{325, 182069},{326, -292517},{327, -875997},{328, 981949},{329, -882097},{330, 675926},{331, 666826},{332, 384951},{333, -203414},{334, 707805},{335, -609986},{336, -991063},{337, 660233},{338, -612818},{339, 289687},{340, 758905},
        {341, -776464},{342, 634052},{343, -285301},{344, -482102},{345, -112724},{346, -543521},{347, 151447},{348, 698105},{349, 289433},{350, -206769},{351, -101793},{352, 661213},{353, -497776},{354, -417620},{355, 859779},{356, 161830},{357, 551209},{358, 164678},{359, 386705},{360, -280225},
        {361, 14763},{362, -397348},{363, -760008},{364, -493734},{365, 283827},{366, -350653},{367, -705470},{368, 382836},{369, 248099},{370, -484369},{371, 44998},{372, -780317},{373, -747298},{374, 876450},{375, 26367},{376, 279235},{377, 456656},{378, 56155},{379, 739793},{380, -409182},
        {381, 560085},{382, 759245},{383, -284391},{384, -190491},{385, 808930},{386, 754152},{387, 543831},{388, -176221},{389, -240610},{390, 938223},{391, 141554},{392, 42254},{393, 198099},{394, 646081},{395, 529779},{396, -736066},{397, 233239},{398, -399642},{399, -37208},{400, 506229},
        {401, 364271},{402, 13252},{403, 310483},{404, -902578},{405, 573493},{406, -389312},{407, 373714},{408, -486329},{409, 1039857},{410, 215771},{411, -582196},{412, 155672},{413, -10251},{414, 713142},{415, 302641},{416, 651946},{417, 41003},{418, 614293},{419, 613367},{420, -442562},
        {421, 309594},{422, 609694},{423, -619467},{424, 747666},{425, 757894},{426, -743423},{427, 271001},{428, 118228},{429, -75373},{430, 269067},{431, 702417},{432, 162866},{433, 664518},{434, 969950},{435, 175980},{436, 250403},{437, -546979},{438, -898943},{439, 444583},{440, -649491},
        {441, -232915},{442, 692288},{443, 259032},{444, 570816},{445, 639733},{446, -242086},{447, 102213},{448, 992555},{449, 454676},{450, 764511},{451, 755801},{452, 488565},{453, 365425},{454, -911267},{455, 379565},{456, 88639},{457, 570691},{458, -954539},{459, -511609},{460, 672180},
        {461, -781019},{462, 749740},{463, -280749},{464, 675532},{465, 22190},{466, 600148},{467, -429009},{468, -316554},{469, -63227},{470, 287104},{471, 611000},{472, 39233},{473, 222181},{474, 818694},{475, 1001986},{476, -686100},{477, -762080},{478, 48368},{479, 381501},{480, 699180},
        {481, -267664},{482, 372914},{483, -206245},{484, -27531},{485, 53050},{486, -224850},{487, 202745},{488, 959906},{489, -382806},{490, -250315},{491, 16306},{492, -332303},{493, 781727},{494, 583400},{495, -670383},{496, 96260},{497, 317272},{498, 710629},{499, 257789},{500, 627306},
        {501, 378178},{502, 232260},{503, -199227},{504, 177838},{505, -628755},{506, 708116},{507, 37523},{508, 467316},{509, 868420},{510, -141305},{511, 377002},{512, -289616},{513, -25889},{514, -101313},{515, -683384},{516, -541556},{517, -722218},{518, -396281},{519, 732250},{520, 128905},
        {521, 769841},{522, 573406},{523, 605541},{524, 743191},{525, -118636},{526, -596029},{527, 491314},{528, 200419},{529, 261132},{530, 39252},{531, -341356},{532, -874328},{533, -305874},{534, -641772},{535, -708597},{536, -823352},{537, 595989},{538, 146398},{539, 548029},{540, -911},
        {541, 468605},{542, 824069},{543, -916554},{544, -670621},{545, -354147},{546, -642930},{547, 757856},{548, -508778},{549, 545687},{550, -562130},{551, -612619},{552, -700773},{553, -929143},{554, 730808},{555, 325640},{556, 525286},{557, 315474},{558, 440013},{559, -399986},{560, 18475},
        {561, 587632},{562, 13144},{563, -123091},{564, -154122},{565, -466893},{566, -432587},{567, -12120},{568, 376466},{569, 498385},{570, -332567},{571, 682719},{572, -317574},{573, 333256},{574, 847908},{575, -559617},{576, -353915},{577, -412948},{578, -307720},{579, -421598},{580, 667337},
        {581, 527762},{582, -338737},{583, 70964},{584, -280467},{585, 87462},{586, -129005},{587, 321900},{588, 131503},{589, -732600},{590, 349939},{591, 779498},{592, 860111},{593, -640488},{594, 717268},{595, 628129},{596, 623012},{597, 23310},{598, 1832},{599, -404753},{600, 622542},
        {601, 591200},{602, 732096},{603, -532066},{604, -659386},{605, -576884},{606, -539315},{607, -342303},{608, 488563},{609, 644607},{610, -159841},{611, 781081},{612, 621139},{613, -302909},{614, -152172},{615, -535671},{616, 724361},{617, -953375},{618, 364232},{619, -676550},{620, 395410},
        {621, -967006},{622, -474211},{623, -397548},{624, -382144},{625, 481767},{626, -19597},{627, -623580},{628, 786052},{629, 72636},{630, 464556},{631, -453455},{632, 184620},{633, 119975},{634, -412740},{635, -395690},{636, 239535},{637, -687271},{638, 392168},{639, 311339},{640, -609217},
        {641, 641947},{642, 303565},{643, -351503},{644, 619765},{645, 36308},{646, 48954},{647, -156380},{648, 585223},{649, 390604},{650, 115658},{651, 662202},{652, -581741},{653, -692429},{654, -133265},{655, -149781},{656, 137569},{657, -299741},{658, 895138},{659, 1020488},{660, -529552},
        {661, 89995},{662, 330303},{663, 303486},{664, -763014},{665, 689964},{666, -134946},{667, 931018},{668, -724376},{669, 908020},{670, 94052},{671, 477182},{672, -639315},{673, -608586},{674, -133258},{675, 143495},{676, 671013},{677, -479852},{678, 177390},{679, -447308},{680, -373675},
        {681, -267834},{682, 520062},{683, 265171},{684, 606859},{685, 366184},{686, 163055},{687, 698599},{688, -275222},{689, -229843},{690, 141972},{691, 381371},{692, -86731},{693, 508740},{694, 610939},{695, -738181},{696, 254976},{697, -596715},{698, -129315},{699, 216032},{700, -169922},
        {701, -274895},{702, 635449},{703, 220787},{704, 595752},{705, 599607},{706, -259950},{707, -115468},{708, -797636},{709, -356622},{710, -193523},{711, -351060},{712, -221912},{713, 657094},{714, -573361},{715, -499453},{716, -702920},{717, 688025},{718, 300383},{719, -227858},{720, -368115},
        {721, 528905},{722, 595076},{723, -283003},{724, -682203},{725, -1012611},{726, -82103},{727, -830768},{728, 230558},{729, -496578},{730, -371741},{731, 104997},{732, 38058},{733, -745634},{734, 638183},{735, 262571},{736, 917585},{737, 417813},{738, -362994},{739, 676576},{740, -255526},
        {741, 450788},{742, -896682},{743, 736986},{744, 220397},{745, -757962},{746, 655890},{747, 655679},{748, -213850},{749, -289544},{750, 613499},{751, 28083},{752, -617096},{753, -168472},{754, -582118},{755, -68897},{756, -724907},{757, 145731},{758, -974417},{759, -87978},{760, 215932},
        {761, 238650},{762, 467596},{763, -342558},{764, 726317},{765, 127628},{766, 222560},{767, -97786},{768, -683165},{769, 118844},{770, 94605},{771, 953531},{772, -372072},{773, 689007},{774, 748397},{775, 58642},{776, 766090},{777, -540722},{778, -732803},{779, -369146},{780, -318675},
        {781, 739547},{782, -566810},{783, 614352},{784, -610166},{785, 370557},{786, 748733},{787, -789726},{788, -156007},{789, 857053},{790, 591300},{791, -296369},{792, 963985},{793, -422177},{794, -872124},{795, -995851},{796, 786658},{797, 351121},{798, 241834},{799, 582878},{800, 758445},
        {801, 318115},{802, 465717},{803, 113005},{804, 880958},{805, -722534},{806, -566550},{807, 752826},{808, 11126},{809, -536607},{810, 432397},{811, 140840},{812, 692546},{813, -611975},{814, 264488},{815, 493704},{816, -110773},{817, 1046320},{818, -613383},{819, 503936},{820, 287763},
        {821, -218414},{822, 152715},{823, -118837},{824, -446614},{825, -940536},{826, -690152},{827, 29724},{828, -269368},{829, 354751},{830, 587474},{831, -637560},{832, 763962},{833, 170363},{834, 139249},{835, -229820},{836, 369081},{837, -701659},{838, 757632},{839, -355087},{840, -759026},
        {841, -636615},{842, -465511},{843, -292508},{844, 182112},{845, -741461},{846, 73650},{847, 495686},{848, -603645},{849, 770282},{850, 429631},{851, -613230},{852, 610376},{853, 141589},{854, -574268},{855, -255863},{856, -666531},{857, 253309},{858, -179502},{859, 554334},{860, 1044748},
        {861, -753261},{862, 504073},{863, -716116},{864, -469998},{865, -343551},{866, -234218},{867, -741923},{868, 201515},{869, 68388},{870, 369808},{871, 737641},{872, 366963},{873, -902858},{874, 273775},{875, 623524},{876, 658052},{877, 472624},{878, -306968},{879, -612156},{880, 435489},
        {881, -645961},{882, 656236},{883, -285579},{884, -164669},{885, 108498},{886, 491247},{887, -401563},{888, -545635},{889, 303761},{890, 435513},{891, 302020},{892, 4528},{893, -54382},{894, -436992},{895, -5154},{896, 178399},{897, -87368},{898, -209416},{899, 523199},{900, 586282},
        {901, 342900},{902, 73845},{903, -528735},{904, 605065},{905, -1020},{906, 123174},{907, 83192},{908, -737994},{909, -599540},{910, 398273},{911, 198510},{912, -460583},{913, -509990},{914, 619585},{915, -658597},{916, 481452},{917, 378815},{918, -646733},{919, -381888},{920, 138294},
        {921, -504131},{922, 429101},{923, -994489},{924, -465885},{925, -331668},{926, -465487},{927, 69747},{928, -652937},{929, 284129},{930, 444223},{931, 561394},{932, -213985},{933, -1028451},{934, 784144},{935, 407917},{936, 33087},{937, 717627},{938, 262565},{939, 96819},{940, 522124},
        {941, 350500},{942, 715347},{943, 510111},{944, 647675},{945, -392699},{946, -345568},{947, 300396},{948, -292487},{949, 584076},{950, -138763},{951, 90233},{952, -299731},{953, -146329},{954, -354705},{955, 343787},{956, 255409},{957, 977633},{958, -557227},{959, 114046},{960, -319192},
        {961, -43839},{962, -728144},{963, 458557},{964, -396958},{965, 404220},{966, -395006},{967, 902033},{968, 614731},{969, 131572},{970, -692389},{971, 199691},{972, 136317},{973, 774137},{974, 687422},{975, -240751},{976, -265962},{977, -142267},{978, -648138},{979, -378798},{980, -520674},
        {981, 272536},{982, -223916},{983, 379158},{984, 155814},{985, -298621},{986, -141509},{987, 1026903},{988, -300427},{989, 745097},{990, 64144},{991, 503219},{992, 708859},{993, -660862},{994, -478690},{995, 62500},{996, -173830},{997, 538603},{998, 990977},{999, 161830},{1000, 365745},
        {1001, -19773},{1002, -59960},{1003, 409372},{1004, 468818},{1005, 78788},{1006, 145964},{1007, -711717},{1008, -310000},{1009, -663941},{1010, 330954},{1011, 588812},{1012, -257082},{1013, -478732},{1014, 743237},{1015, 720915},{1016, -703120},{1017, -454815},{1018, -469250},{1019, -372331},{1020, -97311},
        {1021, 807709},{1022, 125710},{1023, 773572},{1024, 912592},{1025, 178964},{1026, 140054},{1027, 366778},{1028, -327781},{1029, -721496},{1030, 940626},{1031, 500589},{1032, 67612},{1033, 53240},{1034, -247832},{1035, -171555},{1036, -532893},{1037, -160948},{1038, -351813},{1039, -449348},{1040, 7106},
        {1041, 92599},{1042, -68928},{1043, 91389},{1044, 193940},{1045, 742427},{1046, 523863},{1047, 241838},{1048, -536629},{1049, -47369},{1050, 436121},{1051, 775098},{1052, -139977},{1053, -741875},{1054, -22184},{1055, -321177},{1056, -58134},{1057, -393664},{1058, 681647},{1059, 375909},{1060, -180941},
        {1061, -218811},{1062, 433640},{1063, 673178},{1064, 27383},{1065, 724904},{1066, 598445},{1067, -508838},{1068, 309291},{1069, -380748},{1070, -199228},{1071, -577715},{1072, 109213},{1073, -89604},{1074, 102412},{1075, -415409},{1076, -679153},{1077, 458226},{1078, -440896},{1079, 531620},{1080, 777277},
        {1081, -233732},{1082, 327864},{1083, 525005},{1084, 30716},{1085, -458017},{1086, 654365},{1087, -122005},{1088, 411675},{1089, 553637},{1090, -439383},{1091, -384603},{1092, -418348},{1093, -666881},{1094, 334888},{1095, -304014},{1096, 852590},{1097, 207144},{1098, -366977},{1099, 560183},{1100, -337211},
        {1101, 152926},{1102, -424140},{1103, -47587},{1104, 251725},{1105, 522304},{1106, -234701},{1107, -108187},{1108, -192188},{1109, -195896},{1110, 410737},{1111, -386548},{1112, 71582},{1113, 195597},{1114, 126347},{1115, -334178},{1116, 732843},{1117, -571487},{1118, -732558},{1119, 114979},{1120, 971020},
        {1121, -36844},{1122, 1020462},{1123, 137182},{1124, -461115},{1125, 556591},{1126, 717048},{1127, -989209},{1128, 408518},{1129, 23272},{1130, -811601},{1131, 796984},{1132, 57281},{1133, -501819},{1134, -389235},{1135, -643882},{1136, -804096},{1137, -469124},{1138, -253392},{1139, 592238},{1140, 409503},
        {1141, 474136},{1142, 158598},{1143, -133620},{1144, 442114},{1145, 705942},{1146, 443444},{1147, -454109},{1148, 182334},{1149, -64320},{1150, -377968},{1151, 159430},{1152, 328692},{1153, -346285},{1154, 391672},{1155, -487725},{1156, 274156},{1157, 196313},{1158, -630489},{1159, 21119},{1160, 153238},
        {1161, -168208},{1162, 430394},{1163, 595138},{1164, 83714},{1165, 667444},{1166, 355313},{1167, 259337},{1168, 753176},{1169, 377595},{1170, 152379},{1171, 718872},{1172, -48705},{1173, 81565},{1174, -185623},{1175, -400888},{1176, -999323},{1177, 194767},{1178, -419237},{1179, -631670},{1180, -385748},
        {1181, 1013796},{1182, -739201},{1183, -672545},{1184, 45730},{1185, 70897},{1186, 87931},{1187, -346975},{1188, -324568},{1189, -291907},{1190, 463773},{1191, -4845},{1192, 845841},{1193, -337090},{1194, 780620},{1195, -66212},{1196, -773594},{1197, -27439},{1198, 219557},{1199, -760617},{1200, -150858},
        {1201, -33997},{1202, 445779},{1203, 139133},{1204, -292382},{1205, -75369},{1206, -213230},{1207, -120528},{1208, 379968},{1209, -726403},{1210, -507968},{1211, 2338},{1212, -324447},{1213, -423411},{1214, 292264},{1215, 639592},{1216, 512710},{1217, -122805},{1218, 19799},{1219, 325798},{1220, 569047},
        {1221, 237115},{1222, -178118},{1223, 727995},{1224, -691116},{1225, -711876},{1226, -954016},{1227, -687183},{1228, 956773},{1229, 880808},{1230, -255018},{1231, -927800},{1232, 950538},{1233, -428201},{1234, -209604},{1235, -44742},{1236, 607353},{1237, 33111},{1238, 338563},{1239, 716767},{1240, -299710},
        {1241, 722318},{1242, 577031},{1243, 352945},{1244, 838452},{1245, 14755},{1246, -583880},{1247, -724551},{1248, -166854},{1249, -341062},{1250, -201961},{1251, -508650},{1252, -743769},{1253, -562450},{1254, 387534},{1255, -152167},{1256, 302352},{1257, 732306},{1258, 193712},{1259, -273938},{1260, -413890},
        {1261, -133103},{1262, -757591},{1263, 412894},{1264, 877803},{1265, -637453},{1266, 832129},{1267, -635305},{1268, -447990},{1269, -299913},{1270, -163339},{1271, 122480},{1272, -48359},{1273, 512267},{1274, 763337},{1275, 3841},{1276, -447147},{1277, 499874},{1278, -1036633},{1279, -273519},{1280, 60427},
        {1281, 984287},{1282, 678552},{1283, 187534},{1284, -974483},{1285, -292431},{1286, 838063},{1287, -225618},{1288, -19733},{1289, -617347},{1290, 92198},{1291, -680585},{1292, -648187},{1293, -185642},{1294, 95352},{1295, 289779},{1296, -91040},{1297, -946268},{1298, 572133},{1299, 435080},{1300, 775538},
        {1301, 160729},{1302, 207603},{1303, -444907},{1304, 783273},{1305, -19452},{1306, -707342},{1307, -482615},{1308, 551574},{1309, 93079},{1310, 727949},{1311, -625316},{1312, -562070},{1313, -567010},{1314, 406843},{1315, 753501},{1316, -216337},{1317, 848546},{1318, -471832},{1319, -453922},{1320, -26748},
        {1321, 266975},{1322, 131068},{1323, -163286},{1324, -36148},{1325, 419729},{1326, 161659},{1327, -753948},{1328, -276387},{1329, -637576},{1330, 220770},{1331, 116864},{1332, 505811},{1333, -470459},{1334, -386836},{1335, -776465},{1336, -77822},{1337, 118214},{1338, -693795},{1339, -747938},{1340, 558934},
        {1341, -437665},{1342, -380207},{1343, -601351},{1344, -967128},{1345, 446367},{1346, -737056},{1347, -197442},{1348, 644881},{1349, 119361},{1350, -91782},{1351, -575574},{1352, -678418},{1353, -768547},{1354, -1026213},{1355, 95189},{1356, 925946},{1357, 780136},{1358, -198221},{1359, 57054},{1360, 661631},
        {1361, -38672},{1362, 611190},{1363, -151180},{1364, -270218},{1365, -435037},{1366, 587033},{1367, 159927},{1368, 15564},{1369, 184657},{1370, 111604},{1371, 454796},{1372, -165760},{1373, -198022},{1374, 609192},{1375, -492909},{1376, 694146},{1377, 374804},{1378, 1069},{1379, -345563},{1380, -454260},
        {1381, 863357},{1382, 252072},{1383, 612521},{1384, 423429},{1385, 1037372},{1386, -427659},{1387, 493876},{1388, -542909},{1389, -655845},{1390, 976894},{1391, -260114},{1392, 556557},{1393, 689952},{1394, 80211},{1395, 292468},{1396, 637714},{1397, -332271},{1398, 701140},{1399, 739900},{1400, -293443},
        {1401, 600069},{1402, 543654},{1403, 45919},{1404, 705868},{1405, -491459},{1406, -467202},{1407, -697147},{1408, -393233},{1409, -557049},{1410, -223378},{1411, 297635},{1412, -15173},{1413, -284118},{1414, -439206},{1415, -56351},{1416, 497355},{1417, -766242},{1418, -193092},{1419, -301098},{1420, 351458},
        {1421, 1012869},{1422, -69827},{1423, 963941},{1424, -98845},{1425, -349912},{1426, -754859},{1427, 177940},{1428, 560904},{1429, -415361},{1430, -161971},{1431, -745765},{1432, -651922},{1433, 567402},{1434, -202076},{1435, -10502},{1436, -626332},{1437, 116654},{1438, -754663},{1439, -239651},{1440, 263514},
        {1441, -653624},{1442, -661771},{1443, 310169},{1444, 580052},{1445, -455578},{1446, 293192},{1447, 487686},{1448, 5178},{1449, 512978},{1450, 87800},{1451, -480777},{1452, 330796},{1453, 545558},{1454, -391817},{1455, -498000},{1456, -825617},{1457, -567995},{1458, -336676},{1459, -669687},{1460, -777697},
        {1461, -753581},{1462, -138505},{1463, 99738},{1464, -213588},{1465, 330656},{1466, 362105},{1467, 617939},{1468, -355071},{1469, 274350},{1470, 123965},{1471, 490129},{1472, 629765},{1473, 414623},{1474, -711795},{1475, 244732},{1476, -129347},{1477, -372905},{1478, 729474},{1479, 115210},{1480, -389883},
        {1481, -422752},{1482, 942186},{1483, -377229},{1484, 401658},{1485, -720102},{1486, -716336},{1487, 294407},{1488, 553119},{1489, -205444},{1490, -631766},{1491, 321553},{1492, 141212},{1493, 277487},{1494, 67747},{1495, 218600},{1496, 83671},{1497, 464817},{1498, -353663},{1499, -917060},{1500, -784936},
        {1501, 190336},{1502, 416340},{1503, -390863},{1504, 481951},{1505, 782045},{1506, 883836},{1507, 784805},{1508, -925939},{1509, 207582},{1510, -694097},{1511, -497791},{1512, -317627},{1513, 273762},{1514, -752058},{1515, -459392},{1516, 624173},{1517, 446240},{1518, -125889},{1519, -369762},{1520, 459464},
        {1521, 180197},{1522, 76840},{1523, 707603},{1524, -676952},{1525, 332995},{1526, -550769},{1527, 107264},{1528, -540505},{1529, -372585},{1530, 286289},{1531, 773925},{1532, -133608},{1533, -565817},{1534, -257414},{1535, 31903},{1536, 74130},{1537, -470097},{1538, 547434},{1539, -973670},{1540, -759960},
        {1541, 66569},{1542, -347136},{1543, -34262},{1544, -408321},{1545, -326238},{1546, 120747},{1547, -9405},{1548, 17679},{1549, 52301},{1550, -962618},{1551, -556705},{1552, 254301},{1553, -415262},{1554, 782970},{1555, -537711},{1556, 991108},{1557, 636069},{1558, -123525},{1559, 236496},{1560, 150669},
        {1561, -267710},{1562, 435755},{1563, 83743},{1564, 111825},{1565, 633274},{1566, -57903},{1567, -64236},{1568, -17275},{1569, 763921},{1570, 998515},{1571, -11434},{1572, 236405},{1573, -570602},{1574, -318480},{1575, -284290},{1576, -57082},{1577, 243878},{1578, 61878},{1579, 109528},{1580, 129659},
        {1581, 35590},{1582, -532780},{1583, -229324},{1584, -728915},{1585, -384908},{1586, -106446},{1587, 573427},{1588, -321849},{1589, -1016080},{1590, 990328},{1591, 561204},{1592, 878709},{1593, -446021},{1594, -325401},{1595, -978638},{1596, 78138},{1597, -556370},{1598, -480742},{1599, 714249},{1600, 655831},
        {1601, -204348},{1602, 477890},{1603, 691600},{1604, 90659},{1605, 527526},{1606, -564316},{1607, 1048108},{1608, 180071},{1609, -718428},{1610, -463295},{1611, -95009},{1612, 458838},{1613, -422263},{1614, 534274},{1615, -479891},{1616, 222333},{1617, 459409},{1618, -63736},{1619, 171770},{1620, -784613},
        {1621, -3223},{1622, 33889},{1623, -885668},{1624, 917097},{1625, 107237},{1626, -585449},{1627, -33162},{1628, 178700},{1629, -727203},{1630, 122184},{1631, 766058},{1632, 647873},{1633, -265314},{1634, 155571},{1635, 753245},{1636, 239965},{1637, 904863},{1638, -693028},{1639, 937554},{1640, -174244},
        {1641, -774201},{1642, -48853},{1643, -14577},{1644, -707245},{1645, 427882},{1646, -221836},{1647, 952742},{1648, 133070},{1649, -115383},{1650, 456280},{1651, 782866},{1652, 519446},{1653, 691650},{1654, -275544},{1655, -413783},{1656, -91528},{1657, -493390},{1658, -633861},{1659, -264665},{1660, 497648},
        {1661, -515736},{1662, -304442},{1663, 777199},{1664, -431021},{1665, 23755},{1666, -419273},{1667, -38498},{1668, 719164},{1669, 602732},{1670, -766402},{1671, 565798},{1672, -154967},{1673, -14716},{1674, -444671},{1675, 740931},{1676, 157794},{1677, 737105},{1678, -538254},{1679, 387298},{1680, -265501},
        {1681, -353519},{1682, -727634},{1683, 249810},{1684, -686734},{1685, -588427},{1686, 723263},{1687, 455860},{1688, 253250},{1689, 601773},{1690, -56221},{1691, -120968},{1692, 717311},{1693, 411757},{1694, -343448},{1695, 259625},{1696, -724595},{1697, 93857},{1698, 564355},{1699, -633112},{1700, 714828},
        {1701, 148205},{1702, -460446},{1703, 137369},{1704, 1008590},{1705, 664260},{1706, -302076},{1707, -217219},{1708, -500677},{1709, 497162},{1710, -447776},{1711, -115950},{1712, -530691},{1713, 642141},{1714, 1024745},{1715, 852977},{1716, -609892},{1717, -779462},{1718, 436029},{1719, 61856},{1720, -622936},
        {1721, -222147},{1722, -255396},{1723, 793235},{1724, -391514},{1725, -782736},{1726, 716568},{1727, -897014},{1728, 628081},{1729, 621788},{1730, -36712},{1731, -172818},{1732, 47566},{1733, 317042},{1734, 91667},{1735, -583094},{1736, -992332},{1737, -623760},{1738, 353256},{1739, 86191},{1740, -543867},
        {1741, 174860},{1742, 577727},{1743, -842307},{1744, 320339},{1745, -883793},{1746, -610627},{1747, 734427},{1748, -386514},{1749, -157078},{1750, 275266},{1751, -430144},{1752, -138948},{1753, -321392},{1754, 626997},{1755, 564121},{1756, -462284},{1757, -41478},{1758, -119955},{1759, 364370},{1760, -79707},
        {1761, 357517},{1762, -189533},{1763, 432311},{1764, -31540},{1765, 148090},{1766, -177536},{1767, -338026},{1768, 524507},{1769, -292342},{1770, 82258},{1771, 922409},{1772, 122377},{1773, 303293},{1774, 622858},{1775, -4474},{1776, -728090},{1777, -417127},{1778, 428699},{1779, 54630},{1780, -372520},
        {1781, 712226},{1782, -733330},{1783, -583229},{1784, -569079},{1785, 551050},{1786, 282466},{1787, -142753},{1788, -46531},{1789, -266805},{1790, -709538},{1791, 338404},{1792, 53642},{1793, -27527},{1794, -396660},{1795, -959437},{1796, 367196},{1797, 422073},{1798, -504105},{1799, -51526},{1800, 486405},
        {1801, 130597},{1802, 482481},{1803, -227180},{1804, -78199},{1805, -110689},{1806, -767659},{1807, 173335},{1808, 638252},{1809, 71923},{1810, -447314},{1811, 923374},{1812, -250624},{1813, 586029},{1814, 434805},{1815, -657628},{1816, -684566},{1817, -424974},{1818, -430724},{1819, 722698},{1820, 329201},
        {1821, 496270},{1822, -83416},{1823, 293003},{1824, 711507},{1825, -662842},{1826, -472927},{1827, -25254},{1828, -615181},{1829, 869497},{1830, 598285},{1831, 290240},{1832, 530406},{1833, -528784},{1834, 523210},{1835, 354132},{1836, -231350},{1837, -65609},{1838, 259951},{1839, -675788},{1840, 784172},
        {1841, 79843},{1842, 846286},{1843, -171281},{1844, 493811},{1845, -429963},{1846, 494035},{1847, -734372},{1848, -556580},{1849, -412545},{1850, 440399},{1851, -164970},{1852, 396503},{1853, -334873},{1854, -614422},{1855, -321792},{1856, -585214},{1857, 292462},{1858, -271901},{1859, -772951},{1860, 325941},
        {1861, 654482},{1862, 453689},{1863, -661294},{1864, 225715},{1865, 365169},{1866, 91791},{1867, 162248},{1868, -779545},{1869, 734319},{1870, 614857},{1871, 759044},{1872, -484843},{1873, -787870},{1874, -360373},{1875, 73445},{1876, -10042},{1877, -317422},{1878, -568374},{1879, -193687},{1880, -382873},
        {1881, -322269},{1882, 586883},{1883, 423756},{1884, 312549},{1885, -749667},{1886, 20340},{1887, 619262},{1888, 498076},{1889, 624538},{1890, -25969},{1891, 376909},{1892, 377714},{1893, -344704},{1894, 700041},{1895, 661383},{1896, -511358},{1897, 265298},{1898, -734156},{1899, -468696},{1900, -14487},
        {1901, 92993},{1902, 544529},{1903, -69286},{1904, 1033446},{1905, -41479},{1906, 198879},{1907, 70033},{1908, -379983},{1909, 14447},{1910, 56899},{1911, -441771},{1912, 557170},{1913, 366784},{1914, -905123},{1915, -311990},{1916, 662185},{1917, 611453},{1918, 383506},{1919, 559527},{1920, -436639},
        {1921, 308008},{1922, 52063},{1923, 276171},{1924, -265845},{1925, -720505},{1926, 8568},{1927, 92320},{1928, 718931},{1929, -267988},{1930, -293671},{1931, 37240},{1932, -858549},{1933, 348722},{1934, -674906},{1935, -356002},{1936, -959471},{1937, -117387},{1938, -157106},{1939, 726184},{1940, 30907},
        {1941, 256137},{1942, 237365},{1943, -191662},{1944, -545018},{1945, -320261},{1946, -277933},{1947, 423993},{1948, -522374},{1949, 519213},{1950, -940702},{1951, 713017},{1952, -227200},{1953, 771699},{1954, 650776},{1955, -812273},{1956, -460892},{1957, -53532},{1958, -522169},{1959, -474082},{1960, 739851},
        {1961, -45441},{1962, -113186},{1963, -304191},{1964, -56937},{1965, -429890},{1966, 48887},{1967, 597240},{1968, -124820},{1969, 538183},{1970, 546661},{1971, -431450},{1972, -37370},{1973, 181918},{1974, -618214},{1975, -655215},{1976, 526786},{1977, 73231},{1978, 474255},{1979, 261692},{1980, 607615},
        {1981, 650694},{1982, -49807},{1983, -21836},{1984, -638196},{1985, 98924},{1986, -748533},{1987, 171830},{1988, -140188},{1989, -746519},{1990, -436353},{1991, -735025},{1992, 91839},{1993, 536838},{1994, -76082},{1995, 713245},{1996, -2178},{1997, -213140},{1998, -545118},{1999, 761347},{2000, 749715},
        {2001, 440848},{2002, 179721},{2003, 436490},{2004, 641860},{2005, -531388},{2006, 945381},{2007, 145561},{2008, -726449},{2009, -919100},{2010, -608516},{2011, 102709},{2012, -337362},{2013, 437478},{2014, 608424},{2015, 405436},{2016, -534197},{2017, -737462},{2018, 413653},{2019, 620069},{2020, -986638},
        {2021, 143447},{2022, 750484},{2023, -361273},{2024, -95094},{2025, 108211},{2026, -64195},{2027, 893206},{2028, 577156},{2029, 40533},{2030, 26633},{2031, 492316},{2032, -90907},{2033, 409063},{2034, -38698},{2035, -488218},{2036, 658159},{2037, 307915},{2038, -481691},{2039, -761698},{2040, 128203},
        {2041, -120718},{2042, 719431},{2043, -308305},{2044, -76917},{2045, 328575},{2046, -8943},{2047, 828226},{2048, 262197},{2049, -322551},{2050, -405697},{2051, -77789},{2052, 650284},{2053, -99108},{2054, -418474},{2055, -302498},{2056, -619468},{2057, -579697},{2058, -770596},{2059, 299333},{2060, -643695},
        {2061, -715642},{2062, 36253},{2063, -306568},{2064, 665823},{2065, -578731},{2066, 410830},{2067, 735600},{2068, -255266},{2069, 147773},{2070, 13616},{2071, 654938},{2072, -549312},{2073, -489066},{2074, -110575},{2075, 673658},{2076, 191586},{2077, 510613},{2078, -234162},{2079, 126719},{2080, -504756},
        {2081, -688105},{2082, 104678},{2083, 671316},{2084, -84919},{2085, 292145},{2086, -630682},{2087, -131803},{2088, -268966},{2089, 544649},{2090, 119142},{2091, 320083},{2092, -25521},{2093, 553259},{2094, -383262},{2095, -703204},{2096, -425424},{2097, 565202},{2098, 643287},{2099, -288811},{2100, -623835},
        {2101, -711063},{2102, -247442},{2103, 159253},{2104, 196580},{2105, -243548},{2106, 85313},{2107, -911076},{2108, -18034},{2109, 587536},{2110, -591161},{2111, -118127},{2112, -573767},{2113, 475161},{2114, 812827},{2115, 885505},{2116, 626512},{2117, 173376},{2118, -310443},{2119, 96833},{2120, -733141},
        {2121, -289466},{2122, -176116},{2123, -299769},{2124, -162741},{2125, -316038},{2126, 268817},{2127, 349082},{2128, -106669},{2129, -404711},{2130, -444786},{2131, -370621},{2132, -296202},{2133, 742570},{2134, 660948},{2135, 349623},{2136, 117003},{2137, 726996},{2138, 682090},{2139, 357626},{2140, 44511},
        {2141, 313242},{2142, 81245},{2143, -453797},{2144, -235459},{2145, -690128},{2146, -736953},{2147, -621986},{2148, 585710},{2149, 22429},{2150, -189274},{2151, -436984},{2152, -1043176},{2153, 501002},{2154, 714669},{2155, 522208},{2156, 326849},{2157, 618840},{2158, -12621},{2159, 696990},{2160, -90136},
        {2161, 619428},{2162, 560790},{2163, 259396},{2164, 897744},{2165, 291323},{2166, -656512},{2167, 416350},{2168, -59759},{2169, 49831},{2170, -268767},{2171, -246551},{2172, 53707},{2173, 494810},{2174, -10843},{2175, -211844},{2176, -373385},{2177, 703764},{2178, 517279},{2179, 843898},{2180, -616519},
        {2181, -188131},{2182, 759190},{2183, 666546},{2184, -89571},{2185, 160834},{2186, 327441},{2187, -99594},{2188, 126603},{2189, 251489},{2190, 233389},{2191, 255599},{2192, 881723},{2193, 341605},{2194, -598651},{2195, 600092},{2196, 278521},{2197, 518974},{2198, 5031},{2199, 499399},{2200, -959218},
        {2201, 415251},{2202, -602942},{2203, -1079},{2204, 714339},{2205, 402762},{2206, 876294},{2207, -523626},{2208, -931876},{2209, 223532},{2210, 873941},{2211, 780637},{2212, 449581},{2213, 46382},{2214, 16231},{2215, -547670},{2216, 830624},{2217, -527540},{2218, 493822},{2219, -851145},{2220, 403307},
        {2221, 100911},{2222, -670802},{2223, -978103},{2224, 112179},{2225, 40636},{2226, -524312},{2227, -9488},{2228, 555277},{2229, 175642},{2230, -132658},{2231, 379175},{2232, -468401},{2233, -172551},{2234, 570467},{2235, -878818},{2236, -74694},{2237, 140728},{2238, 329738},{2239, -552065},{2240, -100241},
        {2241, -521570},{2242, 436039},{2243, -498004},{2244, 120475},{2245, 8151},{2246, 5042},{2247, 746919},{2248, -47703},{2249, 485500},{2250, -175680},{2251, 30338},{2252, 545477},{2253, -157782},{2254, 74787},{2255, 522561},{2256, 66873},{2257, 212170},{2258, 532557},{2259, 516352},{2260, 746536},
        {2261, -601096},{2262, 277925},{2263, 391518},{2264, 365811},{2265, -344530},{2266, 828409},{2267, 477438},{2268, -513071},{2269, 739742},{2270, -138558},{2271, 604975},{2272, 312499},{2273, -328998},{2274, 497615},{2275, -711617},{2276, 218890},{2277, 393590},{2278, 778997},{2279, -236478},{2280, -666548},
        {2281, -497299},{2282, 88502},{2283, 593699},{2284, -469990},{2285, 606873},{2286, 336471},{2287, 466209},{2288, 556606},{2289, -565122},{2290, 426971},{2291, 376131},{2292, -579807},{2293, 704380},{2294, -263648},{2295, -265092},{2296, -343023},{2297, 245626},{2298, 329272},{2299, -760951},{2300, -203208},
        {2301, -630886},{2302, -732627},{2303, 19315},{2304, -310793},{2305, 384797},{2306, 69153},{2307, 650639},{2308, -137170},{2309, -126304},{2310, -233317},{2311, -651951},{2312, -838067},{2313, -976013},{2314, 443928},{2315, -740574},{2316, -152901},{2317, 270406},{2318, -491694},{2319, 15419},{2320, -664061},
        {2321, -28495},{2322, -231881},{2323, -30809},{2324, -813404},{2325, -547748},{2326, 420497},{2327, -409252},{2328, -532637},{2329, 611063},{2330, 936772},{2331, -416211},{2332, 767499},{2333, 669558},{2334, 684260},{2335, -618346},{2336, 62804},{2337, 127302},{2338, -168788},{2339, 163298},{2340, -65743},
        {2341, 937616},{2342, -578943},{2343, -629808},{2344, -723926},{2345, -378931},{2346, 681273},{2347, 608208},{2348, -91862},{2349, 414547},{2350, 365744},{2351, 318465},{2352, -448383},{2353, 361494},{2354, 422704},{2355, -232376},{2356, -40975},{2357, -703194},{2358, -63618},{2359, 395550},{2360, 307366},
        {2361, 672981},{2362, -378937},{2363, -85669},{2364, 738878},{2365, -367499},{2366, 755313},{2367, -279486},{2368, 230178},{2369, 523978},{2370, 175893},{2371, 424331},{2372, -444846},{2373, 711683},{2374, -539102},{2375, -318815},{2376, -844641},{2377, 148666},{2378, 1024584},{2379, 574012},{2380, 163340},
        {2381, -534195},{2382, -693832},{2383, 387909},{2384, -787421},{2385, 559059},{2386, 210155},{2387, -216061},{2388, 636481},{2389, 744542},{2390, 910403},{2391, 272933},{2392, 548750},{2393, -606420},{2394, 683824},{2395, 1890},{2396, -168461},{2397, 366343},{2398, -952093},{2399, -62663},{2400, 359256},
        {2401, 291738},{2402, 103938},{2403, 462661},{2404, 181151},{2405, -560785},{2406, -586687},{2407, 587454},{2408, -671321},{2409, -97898},{2410, 422287},{2411, -226013},{2412, 191184},{2413, 580199},{2414, -937601},{2415, 466077},{2416, -141772},{2417, -277686},{2418, -150929},{2419, 388055},{2420, 453704},
        {2421, 708454},{2422, 892795},{2423, -276385},{2424, -757218},{2425, 381017},{2426, -662582},{2427, 49604},{2428, -458445},{2429, 669359},{2430, 239303},{2431, 726497},{2432, -269935},{2433, -46829},{2434, -444175},{2435, 126964},{2436, -34307},{2437, 542213},{2438, 1003171},{2439, -116750},{2440, 296692},
        {2441, 480819},{2442, 687669},{2443, -780413},{2444, 847077},{2445, 588565},{2446, 105351},{2447, 555664},{2448, 993467},{2449, -976443},{2450, -688536},{2451, 888868},{2452, -678474},{2453, -709502},{2454, -747519},{2455, 206476},{2456, -25502},{2457, 585398},{2458, -660197},{2459, -164899},{2460, -386811},
        {2461, -70847},{2462, -108617},{2463, 413521},{2464, 585506},{2465, 339301},{2466, 277827},{2467, -263914},{2468, -473917},{2469, 359764},{2470, 654430},{2471, 860716},{2472, 489941},{2473, 593147},{2474, 614019},{2475, -393148},{2476, -595645},{2477, 199637},{2478, -4996},{2479, -229460},{2480, 759378},
        {2481, 408190},{2482, -161052},{2483, 249605},{2484, -385403},{2485, -730591},{2486, 351531},{2487, -182651},{2488, 922119},{2489, 510926},{2490, 1048193},{2491, -390534},{2492, -689171},{2493, 465716},{2494, 17815},{2495, -615121},{2496, -175057},{2497, 573655},{2498, -320068},{2499, 666405},{2500, -14036},
        {2501, 714177},{2502, 776368},{2503, -202154},{2504, 555063},{2505, 520532},{2506, 14856},{2507, 639007},{2508, -353424},{2509, 775165},{2510, 84770},{2511, -4094},{2512, -272978},{2513, -22871},{2514, 745189},{2515, 262030},{2516, -246895},{2517, 135094},{2518, 40215},{2519, 279901},{2520, 684051},
        {2521, -553795},{2522, 766162},{2523, -784638},{2524, -366169},{2525, -464178},{2526, -163754},{2527, -433164},{2528, 625324},{2529, -816673},{2530, 364651},{2531, -941317},{2532, 724768},{2533, 401130},{2534, 142811},{2535, -1028566},{2536, 516165},{2537, 665861},{2538, 745568},{2539, -199587},{2540, -614848},
        {2541, -501222},{2542, -429463},{2543, 1008325},{2544, -449388},{2545, -386264},{2546, -479403},{2547, 756446},{2548, 58567},{2549, 741604},{2550, -489524},{2551, 628227},{2552, 52843},{2553, -215358},{2554, 439138},{2555, -340853},{2556, 393707},{2557, 772040},{2558, 659294},{2559, 565553},{2560, -34934},
        {2561, 353703},{2562, -145232},{2563, 691965},{2564, -78489},{2565, -107371},{2566, 178515},{2567, 751549},{2568, 953510},{2569, 663432},{2570, -1038563},{2571, -409973},{2572, 329399},{2573, -168677},{2574, 126814},{2575, 121301},{2576, 154386},{2577, 571960},{2578, -266716},{2579, 699478},{2580, -220803},
        {2581, -22023},{2582, -391809},{2583, 171046},{2584, 715544},{2585, -556580},{2586, 900025},{2587, 765915},{2588, -549164},{2589, -14722},{2590, 674585},{2591, -159154},{2592, -507445},{2593, -730521},{2594, -118837},{2595, 532716},{2596, 1016687},{2597, 79142},{2598, -309129},{2599, -457620},{2600, -806460},
        {2601, -356687},{2602, 32243},{2603, 697102},{2604, -318974},{2605, 477190},{2606, 623684},{2607, -937441},{2608, 557991},{2609, 373642},{2610, -356874},{2611, 780970},{2612, 249199},{2613, 366810},{2614, -536061},{2615, 525760},{2616, -769132},{2617, 36054},{2618, -27984},{2619, -700414},{2620, 244518},
        {2621, -751480},{2622, 190995},{2623, -681106},{2624, -900934},{2625, -595643},{2626, -510379},{2627, -693306},{2628, -361494},{2629, 541751},{2630, -746482},{2631, -771003},{2632, 221118},{2633, -335275},{2634, -713781},{2635, 531136},{2636, 221535},{2637, 144796},{2638, -567044},{2639, 234450},{2640, -741273},
        {2641, -727442},{2642, -272914},{2643, -37441},{2644, 151734},{2645, 544677},{2646, 152292},{2647, 233783},{2648, -305238},{2649, -752233},{2650, -486837},{2651, 653039},{2652, -220165},{2653, -760210},{2654, 270702},{2655, 208688},{2656, 45257},{2657, -714287},{2658, -877669},{2659, 191906},{2660, 179592},
        {2661, 591372},{2662, 254108},{2663, -475056},{2664, 138537},{2665, -82946},{2666, 560246},{2667, -730467},{2668, -447140},{2669, 242001},{2670, 1040867},{2671, 808853},{2672, 44855},{2673, -304616},{2674, -356552},{2675, -629866},{2676, -119831},{2677, -469091},{2678, -363083},{2679, 443304},{2680, 1046492},
        {2681, -747992},{2682, -4101},{2683, 236419},{2684, -408452},{2685, 552219},{2686, -579737},{2687, 685222},{2688, -176880},{2689, -105520},{2690, 484181},{2691, 116508},{2692, -326926},{2693, -278517},{2694, -605767},{2695, 434575},{2696, 418782},{2697, -798642},{2698, 732379},{2699, -730078},{2700, 1668},
        {2701, 570627},{2702, 564765},{2703, 225306},{2704, 373864},{2705, -487720},{2706, 372622},{2707, -37591},{2708, -618195},{2709, -220466},{2710, -296896},{2711, 674960},{2712, -416924},{2713, -168308},{2714, 481150},{2715, -68507},{2716, 726021},{2717, -87593},{2718, 223015},{2719, -26712},{2720, -40874},
        {2721, -828522},{2722, 606786},{2723, 620097},{2724, -501278},{2725, 503045},{2726, 107707},{2727, 327348},{2728, -784568},{2729, 230472},{2730, -631317},{2731, 572531},{2732, -216930},{2733, -344620},{2734, -137119},{2735, 397466},{2736, 88601},{2737, -636606},{2738, -35175},{2739, 713427},{2740, 252043},
        {2741, -1043010},{2742, -244163},{2743, -198029},{2744, 485114},{2745, 660943},{2746, 300784},{2747, -446968},{2748, -335235},{2749, 404219},{2750, -367092},{2751, 308785},{2752, -794409},{2753, -708049},{2754, -292938},{2755, -355267},{2756, -471733},{2757, 168724},{2758, -429335},{2759, 599163},{2760, -492257},
        {2761, 607293},{2762, -772486},{2763, -211484},{2764, 94832},{2765, 257795},{2766, 37347},{2767, -382178},{2768, -238919},{2769, 583042},{2770, -440304},{2771, 473404},{2772, 876612},{2773, 447200},{2774, -283303},{2775, 765204},{2776, -364225},{2777, -268555},{2778, 581919},{2779, 742037},{2780, 590202},
        {2781, 15560},{2782, 676134},{2783, -783480},{2784, 603484},{2785, -681862},{2786, 440273},{2787, -241897},{2788, -495269},{2789, 91089},{2790, 364425},{2791, -462980},{2792, 487888},{2793, 255113},{2794, -100538},{2795, -672135},{2796, 137890},{2797, -781683},{2798, 234174},{2799, 289808},{2800, -779335},
        {2801, -101753},{2802, -733276},{2803, -960832},{2804, 486269},{2805, 846258},{2806, -764553},{2807, 775663},{2808, 921461},{2809, -591369},{2810, -207148},{2811, 700784},{2812, -233396},{2813, -787141},{2814, -276127},{2815, -750284},{2816, -577135},{2817, -892619},{2818, 716393},{2819, -623127},{2820, -624304},
        {2821, -40287},{2822, -372767},{2823, 130435},{2824, 300346},{2825, -99814},{2826, 417967},{2827, 424968},{2828, -765502},{2829, -465703},{2830, 17330},{2831, 967504},{2832, -555579},{2833, -648187},{2834, 346731},{2835, 14094},{2836, 521308},{2837, -16927},{2838, 536601},{2839, 99172},{2840, -380224},
        {2841, 223275},{2842, 866798},{2843, 652730},{2844, -636056},{2845, 612520},{2846, 476029},{2847, 907637},{2848, 322709},{2849, -381916},{2850, -472255},{2851, 568423},{2852, 8991},{2853, -281766},{2854, -374321},{2855, 267340},{2856, -13900},{2857, 214517},{2858, -730522},{2859, 372603},{2860, 528176},
        {2861, 770458},{2862, -535302},{2863, 1041122},{2864, 878366},{2865, -358135},{2866, -1003383},{2867, 61206},{2868, 246503},{2869, 437661},{2870, 215017},{2871, -652562},{2872, 164220},{2873, 68262},{2874, 272572},{2875, 267697},{2876, -52637},{2877, 397808},{2878, 418528},{2879, -418177},{2880, 634453},
        {2881, -690107},{2882, -10445},{2883, -213313},{2884, -124953},{2885, -674571},{2886, -41924},{2887, -219775},{2888, 921411},{2889, -985198},{2890, -85695},{2891, 352508},{2892, -192549},{2893, -114111},{2894, 468804},{2895, 579334},{2896, 341767},{2897, -508499},{2898, 512719},{2899, -548351},{2900, 711388},
        {2901, 522025},{2902, 148467},{2903, 291102},{2904, 624399},{2905, 321252},{2906, 98622},{2907, 140012},{2908, 247049},{2909, -424764},{2910, -721952},{2911, -627609},{2912, 698237},{2913, -454193},{2914, -69766},{2915, 532154},{2916, 297392},{2917, 127399},{2918, 774354},{2919, 601694},{2920, 181574},
        {2921, 931061},{2922, -263528},{2923, 77080},{2924, 88985},{2925, 259263},{2926, 481830},{2927, -689237},{2928, 547276},{2929, 535438},{2930, 273976},{2931, 279260},{2932, 237742},{2933, 20779},{2934, -402625},{2935, -437422},{2936, -635826},{2937, 682239},{2938, -284025},{2939, -516345},{2940, 273944},
        {2941, -84216},{2942, -448389},{2943, 144015},{2944, -790701},{2945, -555875},{2946, 28092},{2947, 729745},{2948, -159705},{2949, -267066},{2950, 519588},{2951, -387219},{2952, 173944},{2953, -654617},{2954, 300270},{2955, 102700},{2956, 766412},{2957, 452622},{2958, -220566},{2959, 242781},{2960, 479795},
        {2961, -185692},{2962, 580932},{2963, -239099},{2964, -667312},{2965, 266220},{2966, 144388},{2967, 614331},{2968, 442416},{2969, 43776},{2970, 405494},{2971, 14543},{2972, 51707},{2973, -41273},{2974, -538876},{2975, 923348},{2976, 555857},{2977, 322281},{2978, 80467},{2979, -730121},{2980, 836878},
        {2981, -158344},{2982, -362690},{2983, 330504},{2984, -35429},{2985, -753103},{2986, 185032},{2987, 394197},{2988, 767159},{2989, 463999},{2990, -226064},{2991, -413658},{2992, -192729},{2993, 175729},{2994, 500106},{2995, 390587},{2996, 306869},{2997, -20424},{2998, 665469},{2999, -405164},{3000, 156064},
        {3001, -860974},{3002, -904274},{3003, 363188},{3004, 266531},{3005, -267491},{3006, 270296},{3007, -25241},{3008, 333361},{3009, -743581},{3010, 544507},{3011, 611202},{3012, -221928},{3013, -235481},{3014, -697088},{3015, 563867},{3016, -552291},{3017, -741992},{3018, 339468},{3019, 33553},{3020, -30620},
        {3021, -11274},{3022, -805895},{3023, -484367},{3024, -329023},{3025, -539492},{3026, 506345},{3027, -30305},{3028, -495551},{3029, -254309},{3030, 469538},{3031, 1029530},{3032, 157401},{3033, -438733},{3034, 653939},{3035, -118832},{3036, 953726},{3037, -758667},{3038, -29545},{3039, -639263},{3040, -612122},
        {3041, 126139},{3042, -515071},{3043, 28976},{3044, 353053},{3045, -970197},{3046, -988274},{3047, 466558},{3048, -230134},{3049, 140601},{3050, -321384},{3051, 399619},{3052, 880916},{3053, 368911},{3054, 359194},{3055, 82361},{3056, -418630},{3057, -62783},{3058, -784180},{3059, -639100},{3060, 982662},
        {3061, -288778},{3062, 356759},{3063, -364283},{3064, 664663},{3065, 131918},{3066, 696041},{3067, 597626},{3068, 932593},{3069, 323986},{3070, 201743},{3071, -340221},{3072, 579413},{3073, -262746},{3074, -87185},{3075, 360347},{3076, -1031538},{3077, -666544},{3078, -794632},{3079, -639754},{3080, -356854},
        {3081, 219569},{3082, 383481},{3083, 437638},{3084, 880782},{3085, 412008},{3086, 470633},{3087, 617534},{3088, -754327},{3089, 653620},{3090, -309599},{3091, 68579},{3092, -524852},{3093, 520267},{3094, 376422},{3095, -977689},{3096, 976694},{3097, -592807},{3098, 313142},{3099, 157063},{3100, -971746},
        {3101, 229551},{3102, 155579},{3103, -183641},{3104, 942751},{3105, -601934},{3106, 610150},{3107, 420583},{3108, 47921},{3109, -618424},{3110, -994366},{3111, -176927},{3112, -80519},{3113, 225360},{3114, -651112},{3115, 699311},{3116, -748313},{3117, 571369},{3118, -685836},{3119, -238152},{3120, 401271},
        {3121, -398129},{3122, -262845},{3123, -91383},{3124, -613095},{3125, -400614},{3126, 209462},{3127, -233284},{3128, 279450},{3129, -746904},{3130, -46429},{3131, 684271},{3132, 81155},{3133, 752470},{3134, -165991},{3135, 606372},{3136, 295077},{3137, -20165},{3138, -630987},{3139, 1005857},{3140, -626822},
        {3141, 679553},{3142, 151606},{3143, -137429},{3144, -582061},{3145, 411884},{3146, -241156},{3147, 984602},{3148, 139134},{3149, -9962},{3150, 271378},{3151, 239550},{3152, -131728},{3153, -460083},{3154, -412827},{3155, 681337},{3156, -100481},{3157, -702080},{3158, 713655},{3159, -364430},{3160, 469060},
        {3161, 53720},{3162, -491437},{3163, -83042},{3164, 785684},{3165, -462307},{3166, -717790},{3167, -461629},{3168, -446197},{3169, -159378},{3170, -424120},{3171, -353898},{3172, -563728},{3173, -183926},{3174, -741052},{3175, 657439},{3176, -527566},{3177, 267094},{3178, 535326},{3179, -448447},{3180, -286402},
        {3181, 700579},{3182, 736889},{3183, 534516},{3184, -417717},{3185, 425222},{3186, -528775},{3187, -612547},{3188, -601430},{3189, 577768},{3190, 95069},{3191, -323847},{3192, 128031},{3193, -581607},{3194, -684560},{3195, 751642},{3196, -750794},{3197, -660195},{3198, 355941},{3199, -277797},{3200, -92931},
        {3201, 324141},{3202, -238364},{3203, 42258},{3204, 93041},{3205, 116429},{3206, -326057},{3207, -377845},{3208, 907477},{3209, 99025},{3210, 237005},{3211, 595256},{3212, -575568},{3213, 204091},{3214, -764049},{3215, -785471},{3216, -739378},{3217, -306382},{3218, -41682},{3219, 603658},{3220, -201940},
        {3221, -693078},{3222, 1007087},{3223, -739540},{3224, 329138},{3225, 548120},{3226, -254553},{3227, -472384},{3228, -645342},{3229, 535830},{3230, 521897},{3231, -756269},{3232, 381193},{3233, 612376},{3234, -121732},{3235, 109799},{3236, -627079},{3237, -386912},{3238, -929260},{3239, -368759},{3240, -18148},
        {3241, -942552},{3242, 487878},{3243, -445807},{3244, -303641},{3245, 267760},{3246, -374419},{3247, -199761},{3248, -871771},{3249, -18503},{3250, 61093},{3251, -66379},{3252, 730039},{3253, -551740},{3254, -31323},{3255, 824750},{3256, -691319},{3257, -598246},{3258, 606125},{3259, 48663},{3260, -299042},
        {3261, 456275},{3262, -443544},{3263, 667221},{3264, 116486},{3265, -610004},{3266, -21493},{3267, 61091},{3268, -421165},{3269, 775212},{3270, 250023},{3271, -122678},{3272, 156250},{3273, -278803},{3274, -260719},{3275, 462202},{3276, 599129},{3277, 630519},{3278, -556597},{3279, 236591},{3280, 136513},
        {3281, 722888},{3282, 662750},{3283, -748073},{3284, -188986},{3285, -569743},{3286, 870641},{3287, 763084},{3288, 527013},{3289, -116472},{3290, 586572},{3291, 292818},{3292, -591476},{3293, 502794},{3294, -63015},{3295, -473187},{3296, -84994},{3297, -1115},{3298, -77876},{3299, -882577},{3300, -304953},
        {3301, 71959},{3302, 280641},{3303, -358712},{3304, -588648},{3305, 412877},{3306, -457245},{3307, -378948},{3308, -592163},{3309, -579542},{3310, -731194},{3311, -734528},{3312, -764095},{3313, -326066},{3314, 548397},{3315, -339989},{3316, 413102},{3317, -295225},{3318, -86540},{3319, 86402},{3320, 74120},
        {3321, 536854},{3322, -445140},{3323, 74866},{3324, 756886},{3325, -450637},{3326, -279473},{3327, -955},{3328, 616435},{3329, 598421},{3330, -721521},{3331, -443827},{3332, -427181},{3333, 513826},{3334, 43400},{3335, 596856},{3336, 361700},{3337, -227192},{3338, -766760},{3339, -288259},{3340, -369467},
        {3341, -666016},{3342, -658065},{3343, 760203},{3344, 327863},{3345, -644376},{3346, -345280},{3347, 131084},{3348, 413669},{3349, 8933},{3350, 80949},{3351, 546840},{3352, 17305},{3353, -881149},{3354, -61638},{3355, 63280},{3356, 177434},{3357, -202700},{3358, 637842},{3359, 723186},{3360, 925629},
        {3361, 790851},{3362, 849134},{3363, -785290},{3364, -226574},{3365, -534977},{3366, 376884},{3367, -687158},{3368, -1016893},{3369, -327298},{3370, -785248},{3371, -676534},{3372, -830518},{3373, -65604},{3374, -162250},{3375, -171767},{3376, 785996},{3377, 211891},{3378, -658389},{3379, -279865},{3380, -286065},
        {3381, -131621},{3382, -35559},{3383, 389533},{3384, 345339},{3385, -571662},{3386, -535454},{3387, -153316},{3388, 355847},{3389, 771244},{3390, -218832},{3391, -249674},{3392, -92360},{3393, -892029},{3394, -589372},{3395, -47199},{3396, 107872},{3397, 732975},{3398, 231821},{3399, -299711},{3400, 389972},
        {3401, -670334},{3402, 208122},{3403, 662297},{3404, -901617},{3405, 546092},{3406, 514245},{3407, -99412},{3408, 705202},{3409, 486749},{3410, 404808},{3411, -72392},{3412, 433528},{3413, -255985},{3414, 869396},{3415, 481875},{3416, 579023},{3417, 662313},{3418, -763698},{3419, -212547},{3420, 729144},
        {3421, -361985},{3422, -768790},{3423, -866559},{3424, 419226},{3425, 722874},{3426, 614479},{3427, -363994},{3428, -632707},{3429, -549002},{3430, -958082},{3431, -650439},{3432, -701691},{3433, -60768},{3434, 462499},{3435, 711185},{3436, -249305},{3437, -292983},{3438, -116086},{3439, 611985},{3440, 376667},
        {3441, 665674},{3442, 39398},{3443, 767430},{3444, 78450},{3445, -1039456},{3446, 195398},{3447, 481505},{3448, -618166},{3449, -597763},{3450, 746624},{3451, -618331},{3452, 698772},{3453, 104168},{3454, -108755},{3455, 401508},{3456, 93041},{3457, -475857},{3458, -312454},{3459, -660844},{3460, 179551},
        {3461, 660055},{3462, 640810},{3463, 654374},{3464, 276861},{3465, 45331},{3466, -39647},{3467, 603645},{3468, -132691},{3469, -322704},{3470, -466946},{3471, 310849},{3472, 478235},{3473, -385430},{3474, -283630},{3475, -57759},{3476, 58232},{3477, 647432},{3478, 293179},{3479, -708502},{3480, 338752},
        {3481, 573637},{3482, 352658},{3483, -261808},{3484, -30752},{3485, -519425},{3486, 546690},{3487, 748762},{3488, 418031},{3489, -441188},{3490, 470694},{3491, 324084},{3492, 586205},{3493, -592312},{3494, -216753},{3495, -883075},{3496, -884415},{3497, -108044},{3498, 757932},{3499, -653136},{3500, -275836},
        {3501, 849055},{3502, -816048},{3503, 231484},{3504, 125262},{3505, -181185},{3506, 608205},{3507, -465535},{3508, -15203},{3509, -593045},{3510, -576752},{3511, 337850},{3512, -851185},{3513, -149260},{3514, -365285},{3515, -167494},{3516, -373408},{3517, 579631},{3518, 636113},{3519, 422471},{3520, 83856},
        {3521, 42823},{3522, 301125},{3523, 320339},{3524, 529363},{3525, -106968},{3526, 404918},{3527, 622000},{3528, 546098},{3529, 569731},{3530, 300554},{3531, -27467},{3532, -450865},{3533, 946287},{3534, -346591},{3535, 573202},{3536, 381174},{3537, -439533},{3538, 903351},{3539, 543390},{3540, 759797},
        {3541, -718840},{3542, -2686},{3543, -679686},{3544, -5722},{3545, -557217},{3546, 206051},{3547, 661482},{3548, -331534},{3549, -11742},{3550, -135650},{3551, 72779},{3552, 201954},{3553, -442232},{3554, -122865},{3555, 278865},{3556, -757637},{3557, -702205},{3558, 758215},{3559, 369085},{3560, -627252},
        {3561, -180196},{3562, -553740},{3563, 650397},{3564, 497621},{3565, 316725},{3566, -243107},{3567, 556881},{3568, -883146},{3569, -357176},{3570, -60603},{3571, 28049},{3572, 535236},{3573, -688656},{3574, 458342},{3575, -80101},{3576, 814581},{3577, 685718},{3578, 563397},{3579, 602830},{3580, -559821},
        {3581, -496536},{3582, -327878},{3583, 593073},{3584, -16241},{3585, 597412},{3586, 357606},{3587, -17347},{3588, 1042283},{3589, 387658},{3590, -494521},{3591, 207803},{3592, 354782},{3593, 540626},{3594, 535743},{3595, -120127},{3596, -44497},{3597, -77354},{3598, 459519},{3599, -325118},{3600, 194673},
        {3601, 796843},{3602, -161671},{3603, 688876},{3604, -365867},{3605, 77664},{3606, 531668},{3607, 851937},{3608, -628634},{3609, 656011},{3610, -711187},{3611, 558217},{3612, -510342},{3613, 36301},{3614, 125035},{3615, 388888},{3616, -619884},{3617, 272452},{3618, -599550},{3619, 1001970},{3620, -333116},
        {3621, -90194},{3622, 77493},{3623, 635053},{3624, 33827},{3625, 174424},{3626, -360169},{3627, 134121},{3628, -139823},{3629, 523894},{3630, -702603},{3631, -729574},{3632, 545782},{3633, 285793},{3634, 171296},{3635, -611581},{3636, 792259},{3637, -999428},{3638, 11728},{3639, 353345},{3640, -535782},
        {3641, 885664},{3642, 274140},{3643, -387074},{3644, 172145},{3645, -95934},{3646, 325898},{3647, -410158},{3648, 897142},{3649, -581895},{3650, 596728},{3651, 580507},{3652, -158305},{3653, -404238},{3654, 194306},{3655, -85931},{3656, 77431},{3657, 734325},{3658, -91371},{3659, -268299},{3660, 527026},
        {3661, -108843},{3662, 488265},{3663, 904795},{3664, -911782},{3665, 608062},{3666, 883053},{3667, -149379},{3668, 16533},{3669, -571766},{3670, -604567},{3671, 14492},{3672, 525894},{3673, -617439},{3674, -373994},{3675, 707335},{3676, 883235},{3677, -581825},{3678, -977389},{3679, 119162},{3680, 139759},
        {3681, 216134},{3682, -72742},{3683, 18399},{3684, -58746},{3685, 377732},{3686, 505950},{3687, -556927},{3688, -736830},{3689, -645417},{3690, -577211},{3691, 597179},{3692, 566335},{3693, -469656},{3694, -768080},{3695, -434658},{3696, -169749},{3697, 49209},{3698, -269275},{3699, 660741},{3700, 354348},
        {3701, 547471},{3702, -148446},{3703, 499177},{3704, -605009},{3705, 500043},{3706, 235028},{3707, -785097},{3708, -264887},{3709, -355665},{3710, 683727},{3711, 661024},{3712, 1025813},{3713, 119330},{3714, -384744},{3715, 986769},{3716, -573263},{3717, 79041},{3718, -516816},{3719, -471759},{3720, 371010},
        {3721, -667102},{3722, -164579},{3723, -333952},{3724, -396334},{3725, -686289},{3726, 195040},{3727, -646247},{3728, -138747},{3729, -1026308},{3730, 701418},{3731, 233954},{3732, -1034480},{3733, 143137},{3734, 660967},{3735, -525949},{3736, -294838},{3737, 313545},{3738, -222186},{3739, -106302},{3740, -579999},
        {3741, 682357},{3742, -391702},{3743, 478643},{3744, -682454},{3745, -54727},{3746, 1035520},{3747, -619238},{3748, 102789},{3749, -209208},{3750, 386039},{3751, 320551},{3752, 291327},{3753, 339579},{3754, 599564},{3755, -241146},{3756, 511554},{3757, -168082},{3758, -350538},{3759, 348412},{3760, 769444},
        {3761, -33372},{3762, 40700},{3763, 247252},{3764, -436415},{3765, 546822},{3766, -379064},{3767, 485657},{3768, -552993},{3769, -56495},{3770, 1045605},{3771, -202382},{3772, -723844},{3773, 75146},{3774, -433224},{3775, 618038},{3776, -377179},{3777, 458703},{3778, 217508},{3779, 576653},{3780, -482666},
        {3781, 131511},{3782, 114490},{3783, -251095},{3784, -495125},{3785, 481393},{3786, 373342},{3787, 226759},{3788, -753107},{3789, -619658},{3790, 409834},{3791, 290938},{3792, 569835},{3793, 173950},{3794, -156524},{3795, 121042},{3796, -580219},{3797, -526049},{3798, 160947},{3799, -433924},{3800, -7068},
        {3801, -92986},{3802, -54479},{3803, -79587},{3804, 721697},{3805, -907051},{3806, -828916},{3807, 362867},{3808, 941626},{3809, 80599},{3810, 476497},{3811, -867569},{3812, 767560},{3813, 262239},{3814, 388208},{3815, 262816},{3816, 398391},{3817, 539210},{3818, -236442},{3819, -23768},{3820, 351738},
        {3821, 141534},{3822, 4329},{3823, -729379},{3824, -914264},{3825, -712896},{3826, -446704},{3827, -296060},{3828, -248605},{3829, 52917},{3830, 503098},{3831, -611396},{3832, 439691},{3833, 610190},{3834, -991112},{3835, 679781},{3836, -152375},{3837, 347443},{3838, 496941},{3839, -942754},{3840, -20377},
        {3841, -84785},{3842, -354411},{3843, 990888},{3844, 740497},{3845, -740992},{3846, -689632},{3847, 738918},{3848, -200037},{3849, -300451},{3850, 7103},{3851, 635543},{3852, 177280},{3853, 58178},{3854, -175956},{3855, 442658},{3856, -772193},{3857, 594150},{3858, -624245},{3859, -191410},{3860, 555881},
        {3861, 361042},{3862, 409681},{3863, 610644},{3864, -238874},{3865, -82196},{3866, -194988},{3867, 109271},{3868, 630069},{3869, -66900},{3870, 179903},{3871, -306993},{3872, -377126},{3873, 615393},{3874, 530366},{3875, -959076},{3876, -212379},{3877, -421074},{3878, -172215},{3879, 52573},{3880, 17558},
        {3881, -258372},{3882, -509235},{3883, 561072},{3884, -245406},{3885, 612097},{3886, 721674},{3887, 751768},{3888, -293475},{3889, -462960},{3890, -546156},{3891, 93775},{3892, -651251},{3893, -265872},{3894, 601400},{3895, 753897},{3896, -25075},{3897, -708105},{3898, 714776},{3899, -97063},{3900, -117633},
        {3901, 202348},{3902, -112338},{3903, -297851},{3904, 84604},{3905, 374852},{3906, -478004},{3907, 781246},{3908, 501456},{3909, 385327},{3910, -411053},{3911, -120743},{3912, -454182},{3913, 425384},{3914, -39868},{3915, 134249},{3916, -248335},{3917, -351946},{3918, -149825},{3919, -265695},{3920, 615838},
        {3921, 444022},{3922, 382629},{3923, 65931},{3924, -232624},{3925, -680787},{3926, -146318},{3927, -633897},{3928, 37720},{3929, 455773},{3930, -177044},{3931, 345},{3932, 715948},{3933, -298871},{3934, 323796},{3935, 144674},{3936, 406760},{3937, 134040},{3938, 34048},{3939, 885993},{3940, 232351},
        {3941, -177929},{3942, -122394},{3943, -398666},{3944, 303864},{3945, -74746},{3946, -54663},{3947, -572329},{3948, 559114},{3949, 314722},{3950, -581851},{3951, -44862},{3952, 229521},{3953, -511290},{3954, 7572},{3955, 80844},{3956, 147220},{3957, 384440},{3958, 292994},{3959, -161991},{3960, -282681},
        {3961, -241000},{3962, -622684},{3963, 177175},{3964, -425137},{3965, 215407},{3966, 542294},{3967, -842916},{3968, 519870},{3969, -97577},{3970, -295031},{3971, 105369},{3972, 306150},{3973, -439206},{3974, 321576},{3975, -216509},{3976, 258223},{3977, -213699},{3978, 264290},{3979, -524861},{3980, 677816},
        {3981, 838434},{3982, 392436},{3983, 79743},{3984, -1000246},{3985, -15552},{3986, 722037},{3987, 609323},{3988, -656641},{3989, -584240},{3990, 304746},{3991, -400721},{3992, 566040},{3993, 235133},{3994, 536427},{3995, -738728},{3996, -123386},{3997, -295928},{3998, -417187},{3999, -650306},{4000, -60367},
        {4001, -418331},{4002, 32892},{4003, 200595},{4004, 469114},{4005, -413266},{4006, -193776},{4007, -118043},{4008, -218874},{4009, 345608},{4010, -38783},{4011, 13619},{4012, 537076},{4013, -25210},{4014, -1039492},{4015, 151092},{4016, -569732},{4017, 191408},{4018, -601450},{4019, 109857},{4020, -335141},
        {4021, -2283},{4022, -979756},{4023, -762154},{4024, 207782},{4025, 622483},{4026, 116255},{4027, 100911},{4028, 152112},{4029, -610831},{4030, 639102},{4031, 601087},{4032, -64113},{4033, 260120},{4034, -405081},{4035, -171286},{4036, -117324},{4037, -568288},{4038, -1002049},{4039, -565003},{4040, -404162},
        {4041, -252971},{4042, -636964},{4043, -235743},{4044, 21507},{4045, 285887},{4046, -294834},{4047, 331934},{4048, 402851},{4049, 525916},{4050, -26943},{4051, 880355},{4052, -443126},{4053, 427877},{4054, -655433},{4055, 510407},{4056, -160658},{4057, -267439},{4058, 705918},{4059, -322808},{4060, 50741},
        {4061, 46836},{4062, -786104},{4063, 173661},{4064, 764516},{4065, 465392},{4066, 422946},{4067, -1029264},{4068, 558549},{4069, -235219},{4070, 649642},{4071, 348358},{4072, 132168},{4073, 311471},{4074, -807499},{4075, 433980},{4076, -317524},{4077, 24337},{4078, -576534},{4079, 17917},{4080, 27068},
        {4081, 863213},{4082, 627149},{4083, 387188},{4084, 949433},{4085, -459771},{4086, 157982},{4087, 138933},{4088, -145382},{4089, 186437},{4090, 160713},{4091, 651707},{4092, -274669},{4093, 599018},{4094, -620971},{4095, -378344},{4096, -827015},{4097, -217662},{4098, 63516},{4099, 324622},{4100, -466172},
        {4101, -311160},{4102, -271887},{4103, -718371},{4104, -770701},{4105, -375126},{4106, 125941},{4107, 313836},{4108, 439058},{4109, 675581},{4110, 690142},{4111, -332015},{4112, 519032},{4113, 564052},{4114, -328311},{4115, -192421},{4116, 705188},{4117, -470345},{4118, -547390},{4119, -571945},{4120, -695213},
        {4121, 483368},{4122, -39551},{4123, -95478},{4124, 564278},{4125, -446026},{4126, -89238},{4127, 433242},{4128, -183517},{4129, -343272},{4130, 891327},{4131, 612245},{4132, 885809},{4133, 657696},{4134, 928443},{4135, -573180},{4136, 507610},{4137, 242625},{4138, -644265},{4139, -78217},{4140, -159783},
        {4141, -227690},{4142, -245491},{4143, -561934},{4144, -814053},{4145, -653647},{4146, -500870},{4147, -350893},{4148, 735119},{4149, -628918},{4150, 58586},{4151, -712008},{4152, 233719},{4153, 741513},{4154, -626048},{4155, 526867},{4156, 151115},{4157, 393226},{4158, -276996},{4159, 56462},{4160, 572421},
        {4161, -859186},{4162, 626658},{4163, -817804},{4164, 701258},{4165, -647976},{4166, 608763},{4167, -72041},{4168, -579955},{4169, -489782},{4170, -495171},{4171, 171983},{4172, -248516},{4173, -289260},{4174, 105382},{4175, -506756},{4176, -513969},{4177, 929},{4178, -474378},{4179, -477781},{4180, -312469},
        {4181, 571447},{4182, -747645},{4183, 496579},{4184, -443755},{4185, -585863},{4186, -699771},{4187, -177229},{4188, 251226},{4189, 1015019},{4190, 3935},{4191, 539401},{4192, -318334},{4193, 126327},{4194, -862589},{4195, -367935},{4196, -182329},{4197, 929028},{4198, -512219},{4199, -60964},{4200, -238638},
        {4201, 637384},{4202, -501115},{4203, 291555},{4204, 286096},{4205, 517607},{4206, 532129},{4207, 276810},{4208, 834987},{4209, 130690},{4210, 629332},{4211, 772300},{4212, -481200},{4213, -756654},{4214, 50076},{4215, 673841},{4216, 281908},{4217, -1000335},{4218, 923519},{4219, -78615},{4220, 474239},
        {4221, -352111},{4222, 345884},{4223, 367030},{4224, -322978},{4225, -598096},{4226, -517730},{4227, 647014},{4228, -157164},{4229, 760706},{4230, 80736},{4231, 629568},{4232, -436755},{4233, -817245},{4234, 728151},{4235, -449496},{4236, 72762},{4237, -240816},{4238, -477255},{4239, 398321},{4240, 182771},
        {4241, 215516},{4242, 522237},{4243, -310713},{4244, 329862},{4245, 529526},{4246, 152801},{4247, 708148},{4248, -404654},{4249, 55578},{4250, -141707},{4251, -666463},{4252, -701221},{4253, 534454},{4254, -114787},{4255, 249250},{4256, 273055},{4257, 284312},{4258, -406984},{4259, -490883},{4260, -761739},
        {4261, -570969},{4262, -243233},{4263, 734548},{4264, -539653},{4265, -283596},{4266, -526106},{4267, -268877},{4268, 697601},{4269, -899095},{4270, -13814},{4271, 342355},{4272, -330605},{4273, -711366},{4274, -622109},{4275, -94712},{4276, -753835},{4277, -162538},{4278, 126893},{4279, -665922},{4280, -32519},
        {4281, 753939},{4282, -645364},{4283, -355293},{4284, 726927},{4285, 3545},{4286, 479745},{4287, 617936},{4288, -807940},{4289, -300215},{4290, 457591},{4291, -732693},{4292, 296247},{4293, 332171},{4294, -704708},{4295, 313246},{4296, 292687},{4297, -325690},{4298, -56258},{4299, 432188},{4300, 469052},
        {4301, 120065},{4302, 471430},{4303, 472815},{4304, -145752},{4305, -239728},{4306, 295006},{4307, 156415},{4308, 465724},{4309, 318465},{4310, 701870},{4311, 785626},{4312, 605656},{4313, 616237},{4314, -741043},{4315, 1010923},{4316, 241615},{4317, -304987},{4318, 627283},{4319, -1016508},{4320, 526565},
        {4321, -995966},{4322, 66524},{4323, 320064},{4324, 187327},{4325, -684518},{4326, -251270},{4327, -1039811},{4328, 240594},{4329, 212404},{4330, -320119},{4331, -552982},{4332, 411830},{4333, 274305},{4334, 640624},{4335, 635771},{4336, -551967},{4337, -27682},{4338, 70010},{4339, 322979},{4340, -639860},
        {4341, 194472},{4342, 702906},{4343, 722506},{4344, -331088},{4345, -678199},{4346, 987167},{4347, -440097},{4348, -480665},{4349, 219313},{4350, 234567},{4351, -486532},{4352, 924707},{4353, -613865},{4354, -652410},{4355, 600791},{4356, 924836},{4357, 649550},{4358, 192823},{4359, -705913},{4360, -413362},
        {4361, -694556},{4362, 463329},{4363, -412496},{4364, 181503},{4365, 752999},{4366, -778218},{4367, -192059},{4368, -385249},{4369, -295216},{4370, -48142},{4371, -428518},{4372, 192042},{4373, 544007},{4374, 259221},{4375, -134777},{4376, 345027},{4377, 67367},{4378, 309822},{4379, 76002},{4380, -665359},
        {4381, -687932},{4382, 666135},{4383, 187616},{4384, 779032},{4385, -429128},{4386, -68040},{4387, -403599},{4388, -714433},{4389, 6316},{4390, 9627},{4391, -221879},{4392, -401851},{4393, 720763},{4394, -540938},{4395, 775182},{4396, -410722},{4397, -690440},{4398, 184388},{4399, 592709},{4400, -767930},
        {4401, -616967},{4402, -152833},{4403, 203636},{4404, 824716},{4405, 713071},{4406, 437914},{4407, 92473},{4408, 100290},{4409, -567849},{4410, 477022},{4411, -245286},{4412, -583539},{4413, -402806},{4414, -174985},{4415, -18696},{4416, 344042},{4417, -366972},{4418, 676164},{4419, 231611},{4420, -578657},
        {4421, -25972},{4422, -337001},{4423, 414139},{4424, -546211},{4425, 845080},{4426, -370729},{4427, 351094},{4428, -601136},{4429, 63217},{4430, 1025404},{4431, -338930},{4432, 782851},{4433, -264033},{4434, -463015},{4435, 592882},{4436, 75583},{4437, 90326},{4438, -543933},{4439, 476593},{4440, 259145},
        {4441, 70291},{4442, 387344},{4443, 727818},{4444, 494952},{4445, 666499},{4446, -463941},{4447, -421460},{4448, 141057},{4449, -796730},{4450, -542136},{4451, 262915},{4452, 126522},{4453, -710663},{4454, -639736},{4455, -490977},{4456, -70279},{4457, 501951},{4458, -767712},{4459, -286961},{4460, -103395},
        {4461, -210423},{4462, 689858},{4463, -302632},{4464, -178066},{4465, -16818},{4466, 635768},{4467, 67142},{4468, 1014900},{4469, 924640},{4470, -774487},{4471, -127482},{4472, 491781},{4473, 995157},{4474, -671743},{4475, 125599},{4476, 246298},{4477, -303105},{4478, 285808},{4479, 524624},{4480, -123896},
        {4481, 926072},{4482, -409926},{4483, -740512},{4484, 37701},{4485, -335884},{4486, -6309},{4487, 189977},{4488, -259017},{4489, -429127},{4490, 232452},{4491, 121004},{4492, -757799},{4493, 31675},{4494, 473608},{4495, 35061},{4496, -448439},{4497, 765945},{4498, 388580},{4499, -621095},{4500, 450307},
        {4501, -691219},{4502, 96042},{4503, -75499},{4504, -785365},{4505, -106107},{4506, 957355},{4507, -772276},{4508, 733596},{4509, -62757},{4510, 41502},{4511, 184214},{4512, 697711},{4513, 99189},{4514, 380505},{4515, -114795},{4516, -433580},{4517, 499132},{4518, -75842},{4519, 247440},{4520, 691714},
        {4521, -516318},{4522, 503867},{4523, 593437},{4524, -989173},{4525, 443397},{4526, 462516},{4527, 560504},{4528, -65542},{4529, -549198},{4530, 128002},{4531, -77777},{4532, -311976},{4533, 339763},{4534, 659447},{4535, -471686},{4536, -558566},{4537, 977599},{4538, -534454},{4539, -698218},{4540, -389570},
        {4541, 396702},{4542, 295221},{4543, -446936},{4544, -332644},{4545, -693650},{4546, 981909},{4547, -717599},{4548, 198380},{4549, -620228},{4550, 784352},{4551, 59798},{4552, 367276},{4553, 135930},{4554, -252653},{4555, -580769},{4556, 405440},{4557, -1039575},{4558, 33587},{4559, -786034},{4560, -754994},
        {4561, -938715},{4562, 129784},{4563, 523847},{4564, -628553},{4565, -207368},{4566, -427143},{4567, -698300},{4568, 228316},{4569, 735952},{4570, -158475},{4571, -367584},{4572, 280232},{4573, -746098},{4574, -545903},{4575, -201674},{4576, -154924},{4577, -338779},{4578, -155891},{4579, 107659},{4580, 638083},
        {4581, 290726},{4582, -161997},{4583, -228042},{4584, -675920},{4585, -170849},{4586, 134455},{4587, -454065},{4588, -1048165},{4589, 36655},{4590, -444750},{4591, -579297},{4592, -258363},{4593, -487452},{4594, 780421},{4595, 375671},{4596, -326433},{4597, 445237},{4598, -623401},{4599, -435631},{4600, 590926},
        {4601, 190368},{4602, 672062},{4603, -238979},{4604, 187994},{4605, 780544},{4606, 464183},{4607, -362429},{4608, 170166},{4609, 173062},{4610, -223952},{4611, -570664},{4612, 170829},{4613, -3058},{4614, -1017661},{4615, -782392},{4616, -524775},{4617, -248743},{4618, -601870},{4619, 730702},{4620, -520866},
        {4621, 142567},{4622, -265613},{4623, -522988},{4624, 379732},{4625, 881893},{4626, -788350},{4627, -638720},{4628, -418943},{4629, 453130},{4630, -839712},{4631, -757686},{4632, 759436},{4633, 104399},{4634, 210028},{4635, -111452},{4636, 994017},{4637, 66917},{4638, 389062},{4639, 782477},{4640, -175346},
        {4641, 752762},{4642, -390225},{4643, -240214},{4644, -825606},{4645, -999023},{4646, -396114},{4647, 316317},{4648, -555575},{4649, -700485},{4650, -132435},{4651, -481624},{4652, -547988},{4653, -327359},{4654, -110168},{4655, -307251},{4656, -551964},{4657, 381011},{4658, -341297},{4659, -530814},{4660, 64766},
        {4661, 85979},{4662, -489935},{4663, 720908},{4664, -51057},{4665, -124311},{4666, 213462},{4667, 106629},{4668, -844851},{4669, -591550},{4670, 30880},{4671, 370505},{4672, 236504},{4673, -568099},{4674, -273465},{4675, -742950},{4676, -690},{4677, 906203},{4678, -278265},{4679, 763901},{4680, -155612},
        {4681, 685022},{4682, 180711},{4683, -1004659},{4684, -459436},{4685, 135635},{4686, 620378},{4687, -525901},{4688, 756389},{4689, -217320},{4690, -980367},{4691, 26340},{4692, -754370},{4693, -599965},{4694, 660683},{4695, -499606},{4696, -731174},{4697, -629736},{4698, 568852},{4699, -671632},{4700, -350959},
        {4701, 1039560},{4702, -40403},{4703, -707504},{4704, -748262},{4705, 445515},{4706, 504282},{4707, -692141},{4708, -759085},{4709, 123538},{4710, -489044},{4711, 8365},{4712, 640528},{4713, 5771},{4714, 778661},{4715, 369238},{4716, 732363},{4717, 710817},{4718, 535262},{4719, -327669},{4720, 112590},
        {4721, -618078},{4722, -309780},{4723, 730285},{4724, -378610},{4725, 121126},{4726, 248775},{4727, 714129},{4728, 571492},{4729, 54185},{4730, 657898},{4731, 224939},{4732, -247701},{4733, 608233},{4734, 458640},{4735, -1038731},{4736, -597888},{4737, 127119},{4738, -575726},{4739, -658623},{4740, -114959},
        {4741, 463362},{4742, 413179},{4743, -133615},{4744, 737815},{4745, -208070},{4746, -874591},{4747, -259360},{4748, 113042},{4749, 437537},{4750, -750526},{4751, -933752},{4752, 668519},{4753, -777576},{4754, -425766},{4755, -724938},{4756, 672924},{4757, 679062},{4758, 352406},{4759, -41926},{4760, -394203},
        {4761, -182656},{4762, 666471},{4763, 609933},{4764, 687136},{4765, 49350},{4766, 60483},{4767, -49546},{4768, 865244},{4769, -346285},{4770, -453281},{4771, 733344},{4772, -274165},{4773, 455693},{4774, 663216},{4775, -359097},{4776, 854216},{4777, -777395},{4778, -449989},{4779, -484405},{4780, 671281},
        {4781, 94509},{4782, 477186},{4783, -771999},{4784, 3712},{4785, 501310},{4786, -499629},{4787, 772927},{4788, 716136},{4789, -187834},{4790, -595649},{4791, 225620},{4792, 777054},{4793, 58975},{4794, 633655},{4795, 1036263},{4796, 678568},{4797, -906909},{4798, -244385},{4799, 13851},{4800, 664870},
        {4801, -12360},{4802, -512811},{4803, 653684},{4804, 592466},{4805, 266356},{4806, -450415},{4807, 99260},{4808, -766912},{4809, 155465},{4810, 642005},{4811, -665826},{4812, -346376},{4813, -428419},{4814, -466700},{4815, -795458},{4816, -736825},{4817, -303134},{4818, 442169},{4819, -499525},{4820, 189807},
        {4821, 998402},{4822, 345451},{4823, -239952},{4824, 430339},{4825, 56590},{4826, -407473},{4827, -311763},{4828, -348798},{4829, 264542},{4830, -1374},{4831, -216851},{4832, -1023320},{4833, -566878},{4834, -315200},{4835, -775929},{4836, -578438},{4837, 461463},{4838, 749187},{4839, -266160},{4840, 43234},
        {4841, 32238},{4842, 153126},{4843, -250677},{4844, -42534},{4845, 16050},{4846, 318821},{4847, 233537},{4848, -250412},{4849, -669673},{4850, -599997},{4851, 560641},{4852, -190713},{4853, -473361},{4854, -449995},{4855, -631399},{4856, 203449},{4857, -781301},{4858, 698079},{4859, 670131},{4860, 631260},
        {4861, -90965},{4862, -265478},{4863, -186864},{4864, -659636},{4865, -595404},{4866, 83816},{4867, -205306},{4868, -595216},{4869, -653830},{4870, -586773},{4871, -343933},{4872, -874418},{4873, -108225},{4874, 339019},{4875, -532404},{4876, 243145},{4877, 260855},{4878, 514477},{4879, -163286},{4880, -737343},
        {4881, -546679},{4882, 219032},{4883, 651574},{4884, -808797},{4885, -655142},{4886, 152552},{4887, 404248},{4888, -51923},{4889, -307207},{4890, -27456},{4891, -681183},{4892, -415114},{4893, 425663},{4894, -661591},{4895, -168387},{4896, -183644},{4897, -64309},{4898, 470241},{4899, -198939},{4900, -465934},
        {4901, 773023},{4902, 51115},{4903, -532043},{4904, 701531},{4905, -321928},{4906, 103786},{4907, 198307},{4908, 460637},{4909, 129889},{4910, -93455},{4911, 379128},{4912, 928723},{4913, 85800},{4914, -637811},{4915, -122545},{4916, 266475},{4917, 153991},{4918, -454950},{4919, -436547},{4920, 743743},
        {4921, -627126},{4922, -425203},{4923, -961486},{4924, 366744},{4925, 669280},{4926, 365395},{4927, 31066},{4928, -240005},{4929, -949249},{4930, 303221},{4931, 676874},{4932, -183955},{4933, 654864},{4934, 376402},{4935, 121592},{4936, -525516},{4937, -604001},{4938, 584291},{4939, 208427},{4940, -240442},
        {4941, -405837},{4942, -882637},{4943, 527482},{4944, -296817},{4945, 356781},{4946, 93837},{4947, -316999},{4948, -182457},{4949, 653264},{4950, 278919},{4951, 388936},{4952, -655085},{4953, 359166},{4954, -1563},{4955, -370142},{4956, 279623},{4957, -1040979},{4958, 392384},{4959, 894219},{4960, -406507},
        {4961, -712029},{4962, -370618},{4963, -262671},{4964, 290074},{4965, -437199},{4966, -454358},{4967, -744755},{4968, -133384},{4969, 964460},{4970, 757070},{4971, 548959},{4972, -440522},{4973, 80338},{4974, 985079},{4975, -748201},{4976, 447621},{4977, 474812},{4978, -491943},{4979, -711548},{4980, -518754},
        {4981, 144886},{4982, -17504},{4983, 857822},{4984, -646865},{4985, -540426},{4986, -12423},{4987, -513943},{4988, 603914},{4989, 184818},{4990, -290778},{4991, 701106},{4992, -139445},{4993, -205318},{4994, 782291},{4995, -80564},{4996, -120136},{4997, -762241},{4998, -672162},{4999, -757039},{5000, -222097},
        {5001, 645847},{5002, 766727},{5003, 480351},{5004, 462369},{5005, 132778},{5006, -121670},{5007, -444730},{5008, 613377},{5009, 181080},{5010, -474319},{5011, 171730},{5012, -59412},{5013, -303941},{5014, 293660},{5015, -963673},{5016, -541056},{5017, -370778},{5018, 725410},{5019, 723409},{5020, -785427},
        {5021, 624656},{5022, 13404},{5023, 729659},{5024, 708129},{5025, -597202},{5026, 25656},{5027, -704609},{5028, -519179},{5029, -279335},{5030, 109403},{5031, -601491},{5032, 207411},{5033, 707043},{5034, -317348},{5035, 502218},{5036, 151867},{5037, 46275},{5038, 465222},{5039, -535240},{5040, 763577},
        {5041, -54529},{5042, -751202},{5043, 345484},{5044, 945053},{5045, -260745},{5046, 40117},{5047, -471680},{5048, 612527},{5049, -449985},{5050, -564790},{5051, 692061},{5052, 563473},{5053, -474193},{5054, -453743},{5055, -91924},{5056, -670542},{5057, -321207},{5058, 21900},{5059, -738011},{5060, 399527},
        {5061, -475185},{5062, 668380},{5063, -819481},{5064, -12607},{5065, 395374},{5066, 393509},{5067, 819519},{5068, 180220},{5069, -431465},{5070, -640567},{5071, 435198},{5072, -399110},{5073, -672853},{5074, 339892},{5075, 21543},{5076, -514194},{5077, 514212},{5078, 390018},{5079, -638149},{5080, -452127},
        {5081, 606351},{5082, 583437},{5083, 206813},{5084, -544898},{5085, -97752},{5086, 546084},{5087, -203790},{5088, -608612},{5089, -589693},{5090, -119892},{5091, 366558},{5092, -158515},{5093, -105852},{5094, 458770},{5095, -610600},{5096, -277949},{5097, 286611},{5098, -788011},{5099, -577697},{5100, 552997},
        {5101, 329993},{5102, 820668},{5103, 463438},{5104, 818638},{5105, -336592},{5106, 513780},{5107, -375036},{5108, 645913},{5109, 504806},{5110, -262505},{5111, -89620},{5112, -288449},{5113, 710145},{5114, -672441},{5115, -7340},{5116, -1048475},{5117, -765070},{5118, 259995},{5119, 767053},{5120, -766910},
        {5121, 135057},{5122, -366377},{5123, -1001189},{5124, 110038},{5125, 130922},{5126, -192764},{5127, 373723},{5128, 636007},{5129, -599428},{5130, 676326},{5131, 867180},{5132, -123984},{5133, -59576},{5134, 344775},{5135, -199641},{5136, -239657},{5137, 510188},{5138, 565925},{5139, 673079},{5140, 316710},
        {5141, 127096},{5142, 633738},{5143, -134416},{5144, 373867},{5145, -659004},{5146, -18179},{5147, 549950},{5148, -570194},{5149, 492865},{5150, 653897},{5151, 547622},{5152, -297586},{5153, 107239},{5154, -347959},{5155, 362132},{5156, 235156},{5157, -460253},{5158, 318544},{5159, -1039498},{5160, 706507},
        {5161, 98109},{5162, 737672},{5163, 770707},{5164, 314385},{5165, -82565},{5166, 512514},{5167, -893014},{5168, -54750},{5169, 470622},{5170, -605736},{5171, -152509},{5172, -172437},{5173, -38978},{5174, -729821},{5175, 338163},{5176, -442912},{5177, -249378},{5178, 148335},{5179, 316584},{5180, 169448},
        {5181, 771521},{5182, 8162},{5183, -46783},{5184, 502754},{5185, 278424},{5186, -137449},{5187, -517237},{5188, -730684},{5189, -752201},{5190, 450293},{5191, 651090},{5192, -575378},{5193, -287335},{5194, -770181},{5195, 261221},{5196, 628023},{5197, 159928},{5198, 184725},{5199, 714473},{5200, -458906},
        {5201, -302103},{5202, 288379},{5203, -367230},{5204, -439071},{5205, 513078},{5206, -417355},{5207, -102137},{5208, -233033},{5209, 659702},{5210, -213154},{5211, -682338},{5212, -273941},{5213, 221998},{5214, -436822},{5215, 97529},{5216, -267888},{5217, -933701},{5218, -169809},{5219, -750285},{5220, 100152},
        {5221, -769185},{5222, 322648},{5223, 149408},{5224, 114659},{5225, 124485},{5226, 271302},{5227, -479124},{5228, -426872},{5229, 197285},{5230, -4800},{5231, 293893},{5232, 59997},{5233, -22975},{5234, 352901},{5235, -484220},{5236, -547785},{5237, -243765},{5238, 1034911},{5239, -395911},{5240, 53855},
        {5241, -845939},{5242, -30254},{5243, 202107},{5244, 551964},{5245, -900003},{5246, 503454},{5247, -364404},{5248, 195734},{5249, -661380},{5250, 46489},{5251, -27597},{5252, 764949},{5253, 9056},{5254, 443288},{5255, -292625},{5256, 472074},{5257, 69424},{5258, 532850},{5259, -303761},{5260, 512043},
        {5261, 411051},{5262, 916579},{5263, 273865},{5264, -505688},{5265, -12208},{5266, -306696},{5267, -362696},{5268, -218587},{5269, 391332},{5270, -317907},{5271, 201073},{5272, -91115},{5273, 33244},{5274, 169316},{5275, -386451},{5276, 583993},{5277, -332529},{5278, 465879},{5279, 816642},{5280, 175703},
        {5281, -45095},{5282, -643989},{5283, 346869},{5284, 633586},{5285, -74807},{5286, -258102},{5287, 547063},{5288, 218922},{5289, -332312},{5290, -630937},{5291, -643427},{5292, 223310},{5293, 559989},{5294, 427967},{5295, 242491},{5296, 257617},{5297, 115883},{5298, 300938},{5299, -648475},{5300, 562409},
        {5301, 78232},{5302, 750859},{5303, -748938},{5304, 222990},{5305, -740930},{5306, 449733},{5307, 150539},{5308, -158959},{5309, -257213},{5310, 371476},{5311, 107728},{5312, 615864},{5313, 311470},{5314, 427897},{5315, -382540},{5316, 555518},{5317, -300149},{5318, 781083},{5319, -792783},{5320, 174361},
        {5321, -720911},{5322, -597826},{5323, 420494},{5324, 714421},{5325, -341153},{5326, 235456},{5327, 44346},{5328, 379008},{5329, -14507},{5330, -581077},{5331, -463653},{5332, -445428},{5333, 430799},{5334, -135647},{5335, 110414},{5336, -568272},{5337, -530732},{5338, 399628},{5339, -335572},{5340, 1031214},
        {5341, -360574},{5342, 271315},{5343, 531955},{5344, -163223},{5345, -570198},{5346, -183012},{5347, -565231},{5348, 756515},{5349, -453908},{5350, 170852},{5351, 411018},{5352, -642514},{5353, -38639},{5354, 932889},{5355, 954547},{5356, 203211},{5357, -710524},{5358, 432988},{5359, 454386},{5360, -323483},
        {5361, -155711},{5362, 758245},{5363, 338452},{5364, 668804},{5365, 1019392},{5366, -896282},{5367, 647133},{5368, 53588},{5369, -695923},{5370, -674940},{5371, -273869},{5372, 422604},{5373, 839844},{5374, 391974},{5375, -602302},{5376, 1041741},{5377, -588292},{5378, -450316},{5379, -779761},{5380, -721882},
        {5381, 760275},{5382, -758659},{5383, 136190},{5384, -737705},{5385, -569272},{5386, 723261},{5387, -322259},{5388, 244303},{5389, 229927},{5390, -526566},{5391, -92024},{5392, -319304},{5393, -921459},{5394, 337044},{5395, -57768},{5396, 572295},{5397, 180671},{5398, -205283},{5399, -7938},{5400, -241880},
        {5401, 202941},{5402, -659726},{5403, -454062},{5404, -348755},{5405, -57859},{5406, 429392},{5407, 414555},{5408, -352053},{5409, 1013623},{5410, 574336},{5411, -546311},{5412, 609922},{5413, -587019},{5414, -29735},{5415, 9827},{5416, 661788},{5417, -694611},{5418, -327475},{5419, 1034629},{5420, 227026},
        {5421, 614714},{5422, 628960},{5423, 314145},{5424, -854000},{5425, 21206},{5426, -920093},{5427, -4387},{5428, 651757},{5429, 798223},{5430, 762162},{5431, 886627},{5432, -204798},{5433, 64333},{5434, 48782},{5435, 127788},{5436, -626248},{5437, 581254},{5438, 544012},{5439, -245176},{5440, 619067},
        {5441, -19176},{5442, -545034},{5443, -235922},{5444, 500480},{5445, 87372},{5446, -770926},{5447, 723296},{5448, -187940},{5449, -544880},{5450, -333702},{5451, 14600},{5452, -307846},{5453, 738094},{5454, 369556},{5455, -365868},{5456, -588816},{5457, 601683},{5458, -304698},{5459, -296561},{5460, -458769},
        {5461, 783398},{5462, -549727},{5463, 239476},{5464, -327677},{5465, -18324},{5466, 388679},{5467, 195747},{5468, 622823},{5469, -34441},{5470, 25474},{5471, -51644},{5472, 49080},{5473, 551222},{5474, 209437},{5475, 312493},{5476, -910233},{5477, -591392},{5478, -64086},{5479, -297031},{5480, 357480},
        {5481, -112606},{5482, 492505},{5483, -91551},{5484, -752952},{5485, -113856},{5486, 629083},{5487, 379139},{5488, -773788},{5489, -342343},{5490, 520731},{5491, -269828},{5492, -184695},{5493, -680666},{5494, 777079},{5495, -254159},{5496, 206365},{5497, 250462},{5498, -155589},{5499, 712345},{5500, 186830},
        {5501, -457335},{5502, -723031},{5503, -539111},{5504, -656681},{5505, 51373},{5506, 614055},{5507, -192153},{5508, -149518},{5509, -744219},{5510, -652346},{5511, 191351},{5512, 584757},{5513, -328585},{5514, -109714},{5515, 295354},{5516, -345388},{5517, 119808},{5518, 589173},{5519, 70915},{5520, -466613},
        {5521, -10059},{5522, 173665},{5523, -625569},{5524, -605499},{5525, -624380},{5526, -638653},{5527, 458142},{5528, 780848},{5529, -191072},{5530, 443324},{5531, -675123},{5532, -514069},{5533, -385122},{5534, 955578},{5535, -24722},{5536, -316251},{5537, 496424},{5538, 388850},{5539, 763697},{5540, -874772},
        {5541, 442492},{5542, -704699},{5543, 84514},{5544, 410119},{5545, 649048},{5546, -435749},{5547, -528475},{5548, 752930},{5549, 708191},{5550, 363178},{5551, -587811},{5552, 698144},{5553, 217744},{5554, 550352},{5555, 237334},{5556, -660105},{5557, 75317},{5558, -336236},{5559, 441270},{5560, -91876},
        {5561, -394344},{5562, 299189},{5563, 452481},{5564, -443828},{5565, 521529},{5566, -595878},{5567, -399536},{5568, -775679},{5569, 342701},{5570, -379535},{5571, 197237},{5572, -610844},{5573, -348800},{5574, 35236},{5575, -356074},{5576, -677921},{5577, -198486},{5578, -8482},{5579, 676070},{5580, 248723},
        {5581, -399074},{5582, -612252},{5583, -199537},{5584, 397918},{5585, -939611},{5586, -772784},{5587, -846680},{5588, 173129},{5589, -10359},{5590, -16833},{5591, 400972},{5592, -640727},{5593, 469311},{5594, -491523},{5595, 5627},{5596, -249737},{5597, -134368},{5598, 290512},{5599, -239269},{5600, -539609},
        {5601, -197251},{5602, 562187},{5603, 585361},{5604, -38340},{5605, -571948},{5606, 85202},{5607, -482603},{5608, -256672},{5609, 397515},{5610, 272932},{5611, 229131},{5612, -163185},{5613, -261302},{5614, 609053},{5615, 545706},{5616, 663273},{5617, -680369},{5618, -656302},{5619, 162557},{5620, -736682},
        {5621, 658558},{5622, -577289},{5623, 438148},{5624, 429352},{5625, -300846},{5626, -768654},{5627, 618111},{5628, -597889},{5629, -110047},{5630, 402937},{5631, -591653},{5632, 706865},{5633, -728963},{5634, -640925},{5635, -347869},{5636, -523048},{5637, -1010817},{5638, 738966},{5639, -403515},{5640, 263781},
        {5641, -311884},{5642, -956419},{5643, 917911},{5644, -437209},{5645, 1041170},{5646, -902555},{5647, -118466},{5648, -732093},{5649, 389851},{5650, -78827},{5651, 340382},{5652, 861349},{5653, -42822},{5654, -275678},{5655, -197199},{5656, -739820},{5657, 712166},{5658, -938685},{5659, -184100},{5660, -502913},
        {5661, -140576},{5662, 437555},{5663, -839264},{5664, -1042258},{5665, -20933},{5666, 712702},{5667, -375705},{5668, -641819},{5669, -71236},{5670, 270692},{5671, 671477},{5672, -233877},{5673, -717541},{5674, 568247},{5675, 261781},{5676, 512659},{5677, 119840},{5678, 720281},{5679, 528575},{5680, -594124},
        {5681, 703388},{5682, 268139},{5683, 506334},{5684, -460396},{5685, 249475},{5686, -210738},{5687, 239990},{5688, -453789},{5689, -43885},{5690, 179629},{5691, 578112},{5692, -453923},{5693, -56092},{5694, -24264},{5695, -652580},{5696, 303646},{5697, 705087},{5698, -194830},{5699, -664623},{5700, 567939},
        {5701, 158683},{5702, -47102},{5703, 606022},{5704, -416041},{5705, 651313},{5706, 253189},{5707, 108823},{5708, -361581},{5709, 782800},{5710, 218085},{5711, 503231},{5712, 346772},{5713, -137461},{5714, -237363},{5715, -664932},{5716, -577301},{5717, -492920},{5718, -648475},{5719, 360814},{5720, -81127},
        {5721, 267303},{5722, 933887},{5723, -405062},{5724, -368356},{5725, 705702},{5726, 323222},{5727, 899125},{5728, 672179},{5729, -66693},{5730, 603960},{5731, 301202},{5732, 517011},{5733, 209193},{5734, -65240},{5735, -765084},{5736, -342846},{5737, 168341},{5738, 260612},{5739, 523526},{5740, -27582},
        {5741, 250168},{5742, 648192},{5743, -32878},{5744, -603765},{5745, 926118},{5746, -660895},{5747, 564622},{5748, 175270},{5749, 387383},{5750, 454816},{5751, 314810},{5752, 598159},{5753, -1034100},{5754, 457996},{5755, 346391},{5756, -731981},{5757, -927107},{5758, 39647},{5759, 413662},{5760, 1042975},
        {5761, 250386},{5762, 169815},{5763, -251095},{5764, -377893},{5765, -508295},{5766, -721311},{5767, -473758},{5768, -452349},{5769, 828308},{5770, -763046},{5771, 94463},{5772, 617732},{5773, -389308},{5774, 977489},{5775, -452290},{5776, -434721},{5777, 54466},{5778, -78413},{5779, 223357},{5780, 680872},
        {5781, -420174},{5782, -524567},{5783, -482397},{5784, -74655},{5785, -1032544},{5786, -362309},{5787, -942237},{5788, 161474},{5789, -15490},{5790, -382233},{5791, -440195},{5792, 701782},{5793, 782767},{5794, 384358},{5795, -13139},{5796, 599771},{5797, -524140},{5798, -807835},{5799, -696062},{5800, -647440},
        {5801, 562324},{5802, -702965},{5803, 368234},{5804, -245333},{5805, 611629},{5806, 585738},{5807, 476717},{5808, 489074},{5809, -178333},{5810, -531568},{5811, -731764},{5812, -612345},{5813, -361397},{5814, 220316},{5815, -398202},{5816, -105724},{5817, -972765},{5818, 141955},{5819, -385928},{5820, -412962},
        {5821, -717261},{5822, 441247},{5823, 292846},{5824, -481322},{5825, 148470},{5826, 772035},{5827, 440467},{5828, -108034},{5829, 114974},{5830, 141154},{5831, -569711},{5832, 407800},{5833, -383850},{5834, 574252},{5835, -333764},{5836, 121847},{5837, 622979},{5838, -750757},{5839, 481837},{5840, -430331},
        {5841, 220251},{5842, -192238},{5843, -167150},{5844, -756179},{5845, 214738},{5846, -317105},{5847, -713386},{5848, 414533},{5849, 806705},{5850, -412128},{5851, 627461},{5852, 427652},{5853, -219719},{5854, -345412},{5855, 307065},{5856, -421464},{5857, 319745},{5858, 425996},{5859, -506444},{5860, 524634},
        {5861, -745312},{5862, 210206},{5863, -629722},{5864, 2134},{5865, -28921},{5866, -560806},{5867, -763760},{5868, 84722},{5869, -318448},{5870, 823146},{5871, 64229},{5872, -536870},{5873, -680184},{5874, 59227},{5875, 6476},{5876, -55104},{5877, 581137},{5878, -521121},{5879, 494135},{5880, 674924},
        {5881, 757856},{5882, 91800},{5883, 711994},{5884, 649},{5885, 537755},{5886, 617361},{5887, -131089},{5888, -443001},{5889, 556075},{5890, 403771},{5891, -157107},{5892, 679048},{5893, -580470},{5894, -260247},{5895, -449001},{5896, 199814},{5897, -848927},{5898, 9825},{5899, -68507},{5900, -652683},
        {5901, 655526},{5902, 99359},{5903, -668028},{5904, -503171},{5905, -702914},{5906, -602497},{5907, 610145},{5908, -514774},{5909, -597229},{5910, -578327},{5911, -705168},{5912, 931802},{5913, 678708},{5914, 109349},{5915, 394225},{5916, -21388},{5917, -119048},{5918, -149808},{5919, -564648},{5920, -443615},
        {5921, -464484},{5922, -295075},{5923, -455068},{5924, 702039},{5925, 39931},{5926, 685564},{5927, -48554},{5928, -366710},{5929, -853981},{5930, 751173},{5931, 512364},{5932, 665377},{5933, -120218},{5934, 749247},{5935, -833819},{5936, -190194},{5937, 46691},{5938, -930563},{5939, 679425},{5940, -16958},
        {5941, -110068},{5942, -331605},{5943, -713942},{5944, -233266},{5945, -649185},{5946, -579364},{5947, -418635},{5948, 42762},{5949, -267427},{5950, 220150},{5951, 973232},{5952, -279990},{5953, -247783},{5954, 432179},{5955, 778045},{5956, 712112},{5957, -411347},{5958, 937628},{5959, 409905},{5960, -479118},
        {5961, 712735},{5962, -96753},{5963, 902837},{5964, 242813},{5965, -245496},{5966, -302783},{5967, -36088},{5968, -966647},{5969, -519860},{5970, 270806},{5971, -698274},{5972, 642221},{5973, -189152},{5974, -501165},{5975, 310771},{5976, 667805},{5977, -290966},{5978, 746415},{5979, -887739},{5980, 348956},
        {5981, 168475},{5982, -567809},{5983, -668286},{5984, -369266},{5985, -555539},{5986, -179636},{5987, -196558},{5988, -768027},{5989, 519638},{5990, 449789},{5991, -716250},{5992, 505157},{5993, -556177},{5994, 110180},{5995, 261560},{5996, -68694},{5997, -465332},{5998, 853448},{5999, -706315},{6000, 373981},
        {6001, 1003282},{6002, 125750},{6003, -934116},{6004, 685065},{6005, -699884},{6006, -629614},{6007, 509035},{6008, 135235},{6009, -51029},{6010, -11099},{6011, 84444},{6012, 252781},{6013, -298606},{6014, 518696},{6015, -197766},{6016, 534892},{6017, 529585},{6018, 678974},{6019, -418891},{6020, -610570},
        {6021, 532467},{6022, -20042},{6023, -366950},{6024, -363863},{6025, -220316},{6026, 797042},{6027, -189157},{6028, -429889},{6029, -681346},{6030, 765907},{6031, 144162},{6032, -324377},{6033, -355435},{6034, -448162},{6035, 532342},{6036, 502791},{6037, -28896},{6038, 776916},{6039, -474399},{6040, -146841},
        {6041, 569149},{6042, -116464},{6043, 534476},{6044, 638013},{6045, -171783},{6046, -63568},{6047, 960543},{6048, 254166},{6049, -724653},{6050, 741559},{6051, -450114},{6052, -280169},{6053, 210549},{6054, -8980},{6055, 642848},{6056, 609084},{6057, 5648},{6058, -896470},{6059, 125675},{6060, 794048},
        {6061, -124294},{6062, -194128},{6063, 371357},{6064, -177143},{6065, 71991},{6066, 436602},{6067, 446533},{6068, 565830},{6069, -971060},{6070, 239825},{6071, 553420},{6072, 620480},{6073, -385360},{6074, 39498},{6075, -1857},{6076, 363999},{6077, -557079},{6078, -119380},{6079, 124482},{6080, 533421},
        {6081, -733882},{6082, 489395},{6083, -214027},{6084, -707850},{6085, 807430},{6086, 6224},{6087, -216707},{6088, 215463},{6089, -43823},{6090, 403084},{6091, -341944},{6092, -496349},{6093, -169103},{6094, -598672},{6095, -622482},{6096, 94846},{6097, -230084},{6098, 288802},{6099, -781433},{6100, 329},
        {6101, -701322},{6102, -703536},{6103, -656816},{6104, 727615},{6105, 68098},{6106, -178401},{6107, 95368},{6108, 795948},{6109, 313133},{6110, 34841},{6111, -764716},{6112, -719184},{6113, 549257},{6114, -242820},{6115, 179713},{6116, -724778},{6117, 172450},{6118, -67174},{6119, 772688},{6120, 521394},
        {6121, 927784},{6122, -669345},{6123, -194144},{6124, 797690},{6125, -245112},{6126, 774741},{6127, -83320},{6128, 734904},{6129, -297557},{6130, 18670},{6131, -772057},{6132, 387776},{6133, -181480},{6134, 85131},{6135, 1040837},{6136, -772822},{6137, 430319},{6138, -510020},{6139, 300020},{6140, -223730},
        {6141, 88336},{6142, 30341},{6143, -709742},{6144, -763738},{6145, 274299},{6146, -490255},{6147, 427621},{6148, 281623},{6149, 688915},{6150, 249680},{6151, -610547},{6152, 487513},{6153, 627368},{6154, 579809},{6155, 111952},{6156, -127453},{6157, -41988},{6158, 893326},{6159, 536224},{6160, 155015},
        {6161, 865725},{6162, -736266},{6163, -254361},{6164, 591822},{6165, -722789},{6166, -645590},{6167, -266440},{6168, -166013},{6169, -612126},{6170, 520419},{6171, -733193},{6172, -518233},{6173, -41450},{6174, -579689},{6175, 430544},{6176, 244527},{6177, 814623},{6178, 603532},{6179, -623221},{6180, 126832},
        {6181, 573275},{6182, -152556},{6183, 13523},{6184, 756224},{6185, 648260},{6186, -230936},{6187, -207270},{6188, 526426},{6189, -544461},{6190, -821796},{6191, -284603},{6192, -35732},{6193, -692227},{6194, -77954},{6195, -969647},{6196, -389433},{6197, -276869},{6198, -254037},{6199, 340007},{6200, 241029},
        {6201, 661701},{6202, 520603},{6203, -637647},{6204, 505852},{6205, -382648},{6206, -231962},{6207, -256028},{6208, -157223},{6209, -757082},{6210, 540601},{6211, -1014766},{6212, 227171},{6213, -41069},{6214, 454007},{6215, -550180},{6216, -511509},{6217, -758480},{6218, 696151},{6219, 532859},{6220, 340420},
        {6221, 211953},{6222, -550958},{6223, -547465},{6224, 658100},{6225, 754068},{6226, 46321},{6227, -285546},{6228, -481443},{6229, -666777},{6230, -724106},{6231, -444327},{6232, 313525},{6233, 238976},{6234, -522053},{6235, 771060},{6236, -626197},{6237, 186223},{6238, -44495},{6239, -794870},{6240, 661831},
        {6241, -567623},{6242, 786184},{6243, -275255},{6244, 329715},{6245, 569938},{6246, 266955},{6247, -400670},{6248, -544632},{6249, -746206},{6250, 352784},{6251, 413514},{6252, 377386},{6253, 637308},{6254, 215674},{6255, -226146},{6256, -36353},{6257, 687450},{6258, -275364},{6259, -137620},{6260, 101596},
        {6261, -118668},{6262, 287035},{6263, -538872},{6264, -218600},{6265, -206536},{6266, 410597},{6267, 563243},{6268, 477232},{6269, 929870},{6270, -101982},{6271, 38201},{6272, -388241},{6273, 32836},{6274, -408036},{6275, 265711},{6276, 4988},{6277, -406658},{6278, 61943},{6279, -456297},{6280, 345734},
        {6281, -580569},{6282, -291150},{6283, -349205},{6284, -964971},{6285, -592658},{6286, 463346},{6287, 487952},{6288, 128263},{6289, -501143},{6290, 743263},{6291, -215955},{6292, -278705},{6293, 108593},{6294, 694644},{6295, -505522},{6296, -4750},{6297, 144314},{6298, -634707},{6299, -719967},{6300, 574089},
        {6301, 430457},{6302, -668728},{6303, -528168},{6304, -692128},{6305, 717314},{6306, 284015},{6307, -164354},{6308, -392639},{6309, -993799},{6310, 588304},{6311, -194791},{6312, -464412},{6313, -118316},{6314, 398128},{6315, -789171},{6316, -116614},{6317, 68000},{6318, 103686},{6319, -498282},{6320, -305956},
        {6321, -103441},{6322, -288929},{6323, -313466},{6324, -210119},{6325, -450791},{6326, -414565},{6327, 671251},{6328, 486798},{6329, -86645},{6330, -669747},{6331, 588682},{6332, -535052},{6333, 112548},{6334, -851897},{6335, -735877},{6336, 736855},{6337, 1039908},{6338, -402686},{6339, 78295},{6340, 509703},
        {6341, -692720},{6342, -176145},{6343, 719709},{6344, -267699},{6345, -692129},{6346, 692486},{6347, -511141},{6348, 743860},{6349, -697966},{6350, -55980},{6351, -554521},{6352, -1017785},{6353, -769522},{6354, 851406},{6355, -744104},{6356, -232049},{6357, -60158},{6358, -723082},{6359, 220985},{6360, 785180},
        {6361, -732330},{6362, 300040},{6363, 516984},{6364, -382548},{6365, 563055},{6366, -29841},{6367, 586542},{6368, -294814},{6369, -301834},{6370, -622969},{6371, 515255},{6372, 737376},{6373, 586356},{6374, -891462},{6375, -367399},{6376, 409113},{6377, -154103},{6378, 117162},{6379, -117779},{6380, 836326},
        {6381, 312170},{6382, -99255},{6383, -582068},{6384, -530592},{6385, 751548},{6386, 408824},{6387, -219108},{6388, -64342},{6389, -737387},{6390, 452133},{6391, -474110},{6392, 256707},{6393, -430527},{6394, -426430},{6395, 277493},{6396, -175918},{6397, 878273},{6398, 122303},{6399, 336635},{6400, -65850},
        {6401, 80687},{6402, 33662},{6403, -933187},{6404, -507124},{6405, -350176},{6406, 120021},{6407, 494082},{6408, -585879},{6409, -103804},{6410, 341980},{6411, 64713},{6412, -309591},{6413, 45506},{6414, -75262},{6415, -79316},{6416, 716259},{6417, 753119},{6418, -327268},{6419, -997250},{6420, -763626},
        {6421, -5846},{6422, 384111},{6423, -101503},{6424, 171090},{6425, -138981},{6426, -634622},{6427, 781862},{6428, 63596},{6429, 182418},{6430, 514694},{6431, -936022},{6432, 131841},{6433, 665998},{6434, -20044},{6435, -174908},{6436, 433641},{6437, -94885},{6438, -367629},{6439, 528317},{6440, 125168},
        {6441, -930169},{6442, -554074},{6443, 550926},{6444, 865013},{6445, -695158},{6446, -548325},{6447, -470478},{6448, 426618},{6449, 416784},{6450, -524475},{6451, 181943},{6452, 573697},{6453, 285679},{6454, -580426},{6455, 747682},{6456, -582762},{6457, 449062},{6458, 592308},{6459, 229328},{6460, 369016},
        {6461, -745712},{6462, -611921},{6463, 380643},{6464, 764826},{6465, 883868},{6466, -1013551},{6467, 38512},{6468, -740677},{6469, 667001},{6470, 499816},{6471, 17806},{6472, 717671},{6473, 731206},{6474, 462815},{6475, 981200},{6476, 951085},{6477, -755981},{6478, -357926},{6479, 205478},{6480, 547227},
        {6481, -574768},{6482, 771877},{6483, -95473},{6484, 80942},{6485, -270927},{6486, 163182},{6487, 890877},{6488, -167900},{6489, 21573},{6490, 395062},{6491, -457468},{6492, 543475},{6493, 351053},{6494, -464240},{6495, 1026332},{6496, -1020817},{6497, -597494},{6498, -348416},{6499, 525522},{6500, -550709},
        {6501, 774905},{6502, 759191},{6503, 333715},{6504, 150447},{6505, -430809},{6506, -672288},{6507, -199392},{6508, -64565},{6509, -587182},{6510, 771026},{6511, -495002},{6512, 751206},{6513, 43943},{6514, 533010},{6515, 269014},{6516, 712562},{6517, 120614},{6518, 775035},{6519, -441011},{6520, 627903},
        {6521, 715040},{6522, -525844},{6523, 328752},{6524, -102420},{6525, -680076},{6526, 407590},{6527, 266914},{6528, -839467},{6529, 214365},{6530, 647928},{6531, -251649},{6532, 745251},{6533, 219214},{6534, -26696},{6535, 863146},{6536, -128618},{6537, 182051},{6538, 547663},{6539, -212993},{6540, 583407},
        {6541, -370243},{6542, -210595},{6543, 400945},{6544, 464704},{6545, -341370},{6546, -730826},{6547, 394849},{6548, 520085},{6549, 606663},{6550, -334956},{6551, -303077},{6552, -427136},{6553, 774357},{6554, -59611},{6555, 492034},{6556, -542088},{6557, -783674},{6558, 30370},{6559, 50596},{6560, -190986},
        {6561, 538161},{6562, 239080},{6563, 180108},{6564, 500906},{6565, -410529},{6566, -207478},{6567, -687649},{6568, 44320},{6569, -216777},{6570, 212004},{6571, -562272},{6572, -17957},{6573, 933560},{6574, -147764},{6575, -448673},{6576, -62049},{6577, -583003},{6578, -147764},{6579, -612282},{6580, 55440},
        {6581, 261520},{6582, 441740},{6583, 585069},{6584, -545136},{6585, -459983},{6586, -926317},{6587, -453287},{6588, -378505},{6589, -479111},{6590, -468391},{6591, 193369},{6592, 701557},{6593, -447461},{6594, -469239},{6595, -146682},{6596, -593191},{6597, 560560},{6598, 303417},{6599, 52024},{6600, -638688},
        {6601, 680164},{6602, 432557},{6603, -743130},{6604, -267812},{6605, -593380},{6606, 202388},{6607, 18961},{6608, -470619},{6609, -678198},{6610, -231195},{6611, 655968},{6612, -462362},{6613, 541612},{6614, -340876},{6615, 959648},{6616, -61867},{6617, 819562},{6618, 503571},{6619, -75097},{6620, 128514},
        {6621, -38639},{6622, -312469},{6623, 191696},{6624, -485494},{6625, 336668},{6626, 552445},{6627, 206221},{6628, 50232},{6629, 874848},{6630, -192154},{6631, 96466},{6632, 236011},{6633, -658664},{6634, -906532},{6635, 118179},{6636, 357665},{6637, 183826},{6638, -39047},{6639, -445719},{6640, -1034621},
        {6641, -713915},{6642, -202536},{6643, -681713},{6644, 492155},{6645, -346699},{6646, -707340},{6647, -602432},{6648, 377487},{6649, 52253},{6650, 497375},{6651, -945643},{6652, -297289},{6653, 678586},{6654, 787942},{6655, -192602},{6656, 222612},{6657, -95870},{6658, -369644},{6659, -114979},{6660, 541108},
        {6661, 41974},{6662, -717780},{6663, 446594},{6664, 640502},{6665, 23222},{6666, -619158},{6667, 762783},{6668, -816534},{6669, 298523},{6670, -341089},{6671, -779032},{6672, -682814},{6673, 465940},{6674, 410530},{6675, 565546},{6676, -433866},{6677, 535975},{6678, 703746},{6679, 249329},{6680, -470580},
        {6681, -23757},{6682, 616393},{6683, 506487},{6684, 19570},{6685, -396846},{6686, -827592},{6687, 195270},{6688, 194704},{6689, -267607},{6690, 564381},{6691, 417454},{6692, -548763},{6693, -383516},{6694, -611305},{6695, 678829},{6696, 200549},{6697, -17322},{6698, -287705},{6699, -425954},{6700, 219679},
        {6701, -171560},{6702, -142805},{6703, -628023},{6704, -784141},{6705, 465821},{6706, -103887},{6707, -370543},{6708, -702126},{6709, 371662},{6710, 346736},{6711, 539028},{6712, 236745},{6713, -342471},{6714, 261497},{6715, -23281},{6716, 551287},{6717, -400455},{6718, 21743},{6719, 606328},{6720, -113333},
        {6721, -560238},{6722, -653395},{6723, 61914},{6724, 632337},{6725, 245507},{6726, 840062},{6727, -343365},{6728, 177015},{6729, 319871},{6730, -756459},{6731, 46751},{6732, 105598},{6733, 596553},{6734, -215251},{6735, 504359},{6736, 521049},{6737, -313538},{6738, 13353},{6739, -420375},{6740, 390502},
        {6741, 635242},{6742, 549193},{6743, 236522},{6744, -963896},{6745, -353458},{6746, -393148},{6747, -308636},{6748, 834301},{6749, 274337},{6750, 290007},{6751, -146585},{6752, -134813},{6753, 489671},{6754, 741164},{6755, 767376},{6756, 736352},{6757, -950087},{6758, -441971},{6759, 175794},{6760, -950597},
        {6761, 284737},{6762, 15118},{6763, 86350},{6764, 372706},{6765, 465},{6766, 265551},{6767, 1559},{6768, 720035},{6769, 634610},{6770, -219453},{6771, 999541},{6772, 381707},{6773, -61547},{6774, -540151},{6775, 961537},{6776, 448867},{6777, -546784},{6778, 756950},{6779, 662000},{6780, 47268},
        {6781, 28892},{6782, 549549},{6783, 216427},{6784, -388149},{6785, -159408},{6786, 644300},{6787, 146571},{6788, 904510},{6789, -686501},{6790, 110867},{6791, 555977},{6792, -1007913},{6793, 1027541},{6794, 495104},{6795, 49992},{6796, 685022},{6797, -1602},{6798, 218128},{6799, -575941},{6800, 528779},
        {6801, 51681},{6802, 243318},{6803, 280388},{6804, 339087},{6805, 200065},{6806, -576362},{6807, 123358},{6808, 318646},{6809, 232030},{6810, -517819},{6811, -432247},{6812, -241759},{6813, -367612},{6814, -676535},{6815, 713910},{6816, 773104},{6817, -724158},{6818, 6911},{6819, -221565},{6820, 124901},
        {6821, -389329},{6822, -677077},{6823, 331957},{6824, 159554},{6825, -222950},{6826, -404980},{6827, 586411},{6828, 711622},{6829, -291995},{6830, 259536},{6831, -614999},{6832, 712194},{6833, -161190},{6834, 365649},{6835, 868010},{6836, -511356},{6837, -502761},{6838, 18292},{6839, 384689},{6840, -175046},
        {6841, 628219},{6842, -610481},{6843, 546732},{6844, 128264},{6845, 698417},{6846, 359810},{6847, -776476},{6848, 539361},{6849, -190321},{6850, -465969},{6851, -330155},{6852, -491114},{6853, -970016},{6854, 295156},{6855, -364356},{6856, 358055},{6857, 647376},{6858, -81088},{6859, -503328},{6860, 590518},
        {6861, 310938},{6862, 666329},{6863, 354030},{6864, -247913},{6865, 35979},{6866, 742545},{6867, 135117},{6868, 188800},{6869, 57158},{6870, 999092},{6871, -409720},{6872, -411419},{6873, -444176},{6874, -683898},{6875, -234991},{6876, -347267},{6877, 356392},{6878, -336662},{6879, -217649},{6880, 383263},
        {6881, -585260},{6882, -440429},{6883, -112426},{6884, 750571},{6885, -471357},{6886, -673167},{6887, -721129},{6888, 414754},{6889, -549927},{6890, 88864},{6891, -73760},{6892, -107041},{6893, -393046},{6894, -749424},{6895, -634957},{6896, 64519},{6897, -330172},{6898, 308999},{6899, 328781},{6900, -423123},
        {6901, 797162},{6902, -841396},{6903, -101068},{6904, 214638},{6905, 135873},{6906, 433803},{6907, -575742},{6908, -590791},{6909, 333216},{6910, 516376},{6911, -18923},{6912, 416957},{6913, 349300},{6914, -707025},{6915, 96566},{6916, 178835},{6917, -430711},{6918, 503890},{6919, -521394},{6920, 529225},
        {6921, -566179},{6922, -623698},{6923, 12476},{6924, -601792},{6925, 335351},{6926, 387766},{6927, -712183},{6928, 600081},{6929, -215170},{6930, 431379},{6931, -663017},{6932, -246023},{6933, 281452},{6934, 209805},{6935, 409826},{6936, 32430},{6937, 497305},{6938, 282359},{6939, 719324},{6940, 656887},
        {6941, -483084},{6942, -205081},{6943, 640909},{6944, 538213},{6945, 31041},{6946, 31703},{6947, -504292},{6948, 496562},{6949, 226840},{6950, -464689},{6951, -253391},{6952, 17728},{6953, -138801},{6954, 194714},{6955, 229167},{6956, -141697},{6957, -1025670},{6958, 348022},{6959, 292692},{6960, -369277},
        {6961, -223962},{6962, -332388},{6963, 808640},{6964, 447877},{6965, -234747},{6966, 374406},{6967, 322985},{6968, 16757},{6969, -91061},{6970, 318698},{6971, 342283},{6972, -428262},{6973, -24825},{6974, 528917},{6975, -971973},{6976, -715216},{6977, 459300},{6978, 759178},{6979, -270247},{6980, -485725},
        {6981, 1016123},{6982, 992185},{6983, -981417},{6984, -315243},{6985, 492620},{6986, -55554},{6987, -243692},{6988, -743799},{6989, 278616},{6990, -602715},{6991, 2528},{6992, 153035},{6993, 697861},{6994, 409855},{6995, -750763},{6996, 67892},{6997, 454726},{6998, -754390},{6999, -981917},{7000, -142084},
        {7001, -691718},{7002, -442360},{7003, 748682},{7004, 706583},{7005, -217159},{7006, 224533},{7007, -8595},{7008, -138966},{7009, -396420},{7010, 330010},{7011, -31856},{7012, -622997},{7013, 484096},{7014, 377813},{7015, 698151},{7016, 641418},{7017, 767027},{7018, 382485},{7019, 172974},{7020, -368825},
        {7021, -409283},{7022, -620415},{7023, 291227},{7024, -672225},{7025, 92704},{7026, 14475},{7027, -298609},{7028, 957467},{7029, 207718},{7030, -460544},{7031, -482621},{7032, 269029},{7033, -994981},{7034, 714616},{7035, 204462},{7036, -25681},{7037, -9889},{7038, 282573},{7039, -2334},{7040, -325956},
        {7041, -811234},{7042, 670488},{7043, -573033},{7044, -484104},{7045, -538656},{7046, 219563},{7047, 357620},{7048, -500974},{7049, 357230},{7050, 1011409},{7051, -757192},{7052, -771463},{7053, -396150},{7054, -77167},{7055, 285173},{7056, 398645},{7057, -215144},{7058, 342749},{7059, 742537},{7060, -146082},
        {7061, -33182},{7062, 516668},{7063, 676935},{7064, 669787},{7065, 593934},{7066, 245970},{7067, 488246},{7068, -82999},{7069, -657072},{7070, -99410},{7071, 113435},{7072, -701242},{7073, -677289},{7074, 321674},{7075, 96724},{7076, 500606},{7077, -597506},{7078, -581758},{7079, -100602},{7080, 448884},
        {7081, 589524},{7082, -144125},{7083, 799958},{7084, -53841},{7085, 133784},{7086, -131174},{7087, 17529},{7088, 510239},{7089, -464860},{7090, 540124},{7091, -336119},{7092, 464871},{7093, -619292},{7094, -713312},{7095, -400795},{7096, -9018},{7097, 841693},{7098, -416090},{7099, -380506},{7100, 466560},
        {7101, -423650},{7102, -326495},{7103, 575995},{7104, -180785},{7105, 129712},{7106, -162475},{7107, 295279},{7108, -332720},{7109, 650846},{7110, -383969},{7111, -276817},{7112, -912439},{7113, 770890},{7114, -757469},{7115, 33179},{7116, -231509},{7117, 552614},{7118, -136340},{7119, -185107},{7120, -47398},
        {7121, -482331},{7122, -323566},{7123, -681460},{7124, -579807},{7125, 120751},{7126, -597970},{7127, -503721},{7128, 613357},{7129, 635022},{7130, -205996},{7131, -576503},{7132, 194990},{7133, -1042103},{7134, -324084},{7135, 55498},{7136, -576870},{7137, -382739},{7138, 646594},{7139, 283009},{7140, 157408},
        {7141, -206291},{7142, 767858},{7143, 287079},{7144, -637373},{7145, 379765},{7146, -446928},{7147, -146766},{7148, 950870},{7149, -609125},{7150, 636704},{7151, 416794},{7152, -794072},{7153, -312311},{7154, -739507},{7155, -270711},{7156, -261776},{7157, 785358},{7158, 104224},{7159, 577742},{7160, -527275},
        {7161, 263653},{7162, 362059},{7163, -556907},{7164, -395063},{7165, -753178},{7166, 553683},{7167, -490984},{7168, 690943},{7169, -98569},{7170, -336922},{7171, 184011},{7172, 185021},{7173, -364917},{7174, 279044},{7175, -773675},{7176, 669040},{7177, 476370},{7178, -859439},{7179, -962336},{7180, -367100},
        {7181, 101900},{7182, -746740},{7183, 749581},{7184, 949934},{7185, -738035},{7186, 35166},{7187, 443205},{7188, 19520},{7189, -300888},{7190, 749070},{7191, 958028},{7192, -288463},{7193, -698947},{7194, 138185},{7195, -522286},{7196, 199256},{7197, -520799},{7198, 414375},{7199, -163127},{7200, -887111},
        {7201, -505991},{7202, 396546},{7203, -161811},{7204, -529308},{7205, -184291},{7206, -524808},{7207, 710829},{7208, -717647},{7209, 493150},{7210, 214036},{7211, 749105},{7212, -730626},{7213, 612101},{7214, 354771},{7215, -996367},{7216, -599886},{7217, 694069},{7218, -54},{7219, -374602},{7220, -151179},
        {7221, -705534},{7222, -418321},{7223, 296527},{7224, 98342},{7225, 37278},{7226, -427201},{7227, -158627},{7228, 710322},{7229, 573870},{7230, 640434},{7231, 429580},{7232, -801803},{7233, 327523},{7234, -524750},{7235, -290464},{7236, 39453},{7237, -473636},{7238, 123140},{7239, 93824},{7240, 730843},
        {7241, -918334},{7242, 289872},{7243, -744251},{7244, -750605},{7245, 74694},{7246, 5551},{7247, -655997},{7248, -602801},{7249, -730155},{7250, 586767},{7251, 298611},{7252, -686286},{7253, -988269},{7254, -313984},{7255, -551920},{7256, -801729},{7257, -579960},{7258, 129263},{7259, -658277},{7260, -680427},
        {7261, 771782},{7262, -113804},{7263, -1005668},{7264, 381622},{7265, -308837},{7266, -160880},{7267, 77313},{7268, 410498},{7269, -498883},{7270, -719393},{7271, -509460},{7272, 485984},{7273, 1004864},{7274, 777216},{7275, 825573},{7276, 76046},{7277, 388500},{7278, 219986},{7279, -663807},{7280, 770743},
        {7281, -91871},{7282, 163235},{7283, -443720},{7284, -619868},{7285, -141699},{7286, 724237},{7287, -769493},{7288, 189491},{7289, -720823},{7290, 794808},{7291, 219582},{7292, -132901},{7293, -149510},{7294, -556950},{7295, 779537},{7296, -37162},{7297, -711520},{7298, 452738},{7299, 1025663},{7300, 280185},
        {7301, -389907},{7302, 507310},{7303, -334813},{7304, 34333},{7305, -324814},{7306, -686699},{7307, -728616},{7308, 464599},{7309, 641400},{7310, 75257},{7311, -668365},{7312, -829195},{7313, 433059},{7314, 258071},{7315, -582168},{7316, -179809},{7317, -458699},{7318, 336476},{7319, 357163},{7320, -1042909},
        {7321, -109905},{7322, 480904},{7323, 95101},{7324, -486582},{7325, 46945},{7326, -1012624},{7327, 86855},{7328, -288232},{7329, -201725},{7330, -597641},{7331, -201942},{7332, 404549},{7333, 88624},{7334, 371539},{7335, -763087},{7336, -375194},{7337, -782934},{7338, -121437},{7339, -179684},{7340, 544785},
        {7341, -763646},{7342, 907167},{7343, 271247},{7344, -455782},{7345, 754173},{7346, 153078},{7347, 662341},{7348, -387253},{7349, -172205},{7350, -253263},{7351, 733055},{7352, -596913},{7353, 502991},{7354, -776075},{7355, 132719},{7356, -634525},{7357, -500124},{7358, 321017},{7359, -531160},{7360, -59944},
        {7361, 736779},{7362, -518212},{7363, 712065},{7364, -194113},{7365, 394035},{7366, -645800},{7367, -393075},{7368, 405036},{7369, 710210},{7370, -326023},{7371, -688941},{7372, 475945},{7373, 451459},{7374, 309954},{7375, 503293},{7376, -720109},{7377, -544243},{7378, -315065},{7379, 563406},{7380, 140380},
        {7381, 554853},{7382, 132937},{7383, -540911},{7384, 338947},{7385, 763351},{7386, -1040317},{7387, -96784},{7388, 199858},{7389, 603381},{7390, -694783},{7391, 901477},{7392, -903803},{7393, 393093},{7394, 549774},{7395, 1021151},{7396, 14420},{7397, 322241},{7398, -285023},{7399, -742647},{7400, 382878},
        {7401, 993026},{7402, 750045},{7403, -146566},{7404, 89589},{7405, -379717},{7406, 394719},{7407, 711214},{7408, -448074},{7409, -496229},{7410, -49810},{7411, 820207},{7412, 238275},{7413, 247084},{7414, -514547},{7415, -259560},{7416, -4096},{7417, 707531},{7418, -277450},{7419, -494391},{7420, -260006},
        {7421, -744508},{7422, 371877},{7423, -674396},{7424, -524245},{7425, 132381},{7426, -475863},{7427, 735815},{7428, -257543},{7429, 718568},{7430, -268731},{7431, 56347},{7432, -686982},{7433, 550681},{7434, -554734},{7435, -665807},{7436, -547606},{7437, -228556},{7438, -630617},{7439, 864320},{7440, 396813},
        {7441, 23811},{7442, 777624},{7443, -195497},{7444, -367070},{7445, -752392},{7446, -454240},{7447, 185615},{7448, -284059},{7449, 78437},{7450, 694073},{7451, 661561},{7452, 254924},{7453, 46220},{7454, -135340},{7455, 358306},{7456, -448705},{7457, 126037},{7458, -12562},{7459, -909074},{7460, -408912},
        {7461, -313537},{7462, -318696},{7463, -647695},{7464, 105184},{7465, -892045},{7466, 732943},{7467, -717197},{7468, 235135},{7469, 308888},{7470, -235738},{7471, 309882},{7472, 414819},{7473, -173118},{7474, -760186},{7475, 153608},{7476, -443743},{7477, -592170},{7478, 906490},{7479, -414719},{7480, 282702},
        {7481, -121132},{7482, -430262},{7483, -512892},{7484, 61232},{7485, -395998},{7486, -560967},{7487, 925558},{7488, -467196},{7489, 837539},{7490, 429837},{7491, 227008},{7492, -391832},{7493, -419097},{7494, -323840},{7495, 788561},{7496, -581657},{7497, -330376},{7498, 666068},{7499, -777130},{7500, 314940},
        {7501, 697520},{7502, 175418},{7503, 173810},{7504, 391733},{7505, -699891},{7506, -567354},{7507, 734254},{7508, 361926},{7509, 249021},{7510, -613551},{7511, -39679},{7512, -195473},{7513, 587954},{7514, 677041},{7515, 27221},{7516, -421028},{7517, 567947},{7518, -505697},{7519, -474019},{7520, -121095},
        {7521, 397107},{7522, -642089},{7523, 615316},{7524, 558805},{7525, -36407},{7526, -519507},{7527, -645638},{7528, 272387},{7529, 210513},{7530, 475131},{7531, 568821},{7532, 588270},{7533, 240097},{7534, -601601},{7535, -763139},{7536, -753262},{7537, 27422},{7538, 4304},{7539, -836333},{7540, -186578},
        {7541, -481381},{7542, -471382},{7543, 257339},{7544, -464326},{7545, 681119},{7546, -452728},{7547, -623512},{7548, 347454},{7549, -158891},{7550, -80342},{7551, 959453},{7552, -929786},{7553, 141268},{7554, -478160},{7555, -926942},{7556, 167643},{7557, -483575},{7558, -212146},{7559, -141593},{7560, 976264},
        {7561, 750000},{7562, -211397},{7563, 246188},{7564, -630876},{7565, -836714},{7566, -505928},{7567, 115229},{7568, -433401},{7569, -20810},{7570, 239966},{7571, 324155},{7572, -894511},{7573, 593196},{7574, 194687},{7575, 372608},{7576, -412139},{7577, -932174},{7578, -685543},{7579, 115423},{7580, -490695},
        {7581, -804043},{7582, -431136},{7583, -768520},{7584, -518419},{7585, -769215},{7586, 600551},{7587, 804070},{7588, 337909},{7589, -500795},{7590, 313060},{7591, 495919},{7592, 598224},{7593, 194488},{7594, 117845},{7595, 230525},{7596, -783361},{7597, 250761},{7598, 737305},{7599, -649549},{7600, -502863},
        {7601, 564407},{7602, -163590},{7603, -784415},{7604, -541157},{7605, 159935},{7606, 747193},{7607, 325983},{7608, -733875},{7609, -731591},{7610, -330691},{7611, 334843},{7612, -337629},{7613, -191003},{7614, -101304},{7615, 205371},{7616, -548484},{7617, -531007},{7618, -62546},{7619, -238861},{7620, -442948},
        {7621, -327484},{7622, -643246},{7623, 302012},{7624, 134475},{7625, 307278},{7626, 644302},{7627, 362374},{7628, -138070},{7629, -278807},{7630, 1026495},{7631, 564535},{7632, 216785},{7633, -10750},{7634, 173245},{7635, 559480},{7636, -493387},{7637, -485242},{7638, -681955},{7639, -682996},{7640, 634821},
        {7641, 27684},{7642, 272665},{7643, 159452},{7644, -749749},{7645, -274705},{7646, 644170},{7647, -204424},{7648, -738624},{7649, 354013},{7650, 383176},{7651, -381411},{7652, -260216},{7653, 142846},{7654, -103032},{7655, -403312},{7656, 419147},{7657, 336719},{7658, 157088},{7659, -255699},{7660, 169888},
        {7661, 224135},{7662, 146223},{7663, -501662},{7664, 416163},{7665, 439647},{7666, 990219},{7667, 495023},{7668, 82695},{7669, 324376},{7670, 217259},{7671, -1025596},{7672, 471647},{7673, -164087},{7674, 70515},{7675, 324535},{7676, -372862},{7677, 631635},{7678, -763435},{7679, 865559},{7680, 767403},
        {7681, 594779},{7682, 60023},{7683, -242826},{7684, -368263},{7685, 763039},{7686, -652294},{7687, -919331},{7688, 711231},{7689, 659841},{7690, 785514},{7691, 654334},{7692, 363122},{7693, -324436},{7694, -240868},{7695, -829338},{7696, -565101},{7697, 728242},{7698, 320392},{7699, -103319},{7700, 135408},
        {7701, -174563},{7702, 96518},{7703, 50474},{7704, -588717},{7705, -185712},{7706, -612543},{7707, 410627},{7708, -700474},{7709, -904236},{7710, 364870},{7711, -698676},{7712, 208651},{7713, 760791},{7714, -745535},{7715, 155680},{7716, -400934},{7717, 831696},{7718, 540937},{7719, -337671},{7720, 168611},
        {7721, 539429},{7722, 401348},{7723, -153596},{7724, -157818},{7725, 475305},{7726, -391208},{7727, 240896},{7728, 460954},{7729, -717815},{7730, -215402},{7731, 614060},{7732, -751313},{7733, 165028},{7734, 642977},{7735, 570972},{7736, -269635},{7737, 436532},{7738, 553310},{7739, 475238},{7740, -129663},
        {7741, 1027034},{7742, 677595},{7743, -598006},{7744, 123445},{7745, 464572},{7746, 438812},{7747, -293250},{7748, -720249},{7749, 479851},{7750, -358202},{7751, 878437},{7752, -564577},{7753, 536845},{7754, -780739},{7755, -909460},{7756, -821887},{7757, 965829},{7758, 297316},{7759, -616181},{7760, -332036},
        {7761, -38109},{7762, 461249},{7763, 481876},{7764, 786079},{7765, -455095},{7766, 616244},{7767, 496278},{7768, -585472},{7769, -424922},{7770, 140751},{7771, -684711},{7772, -960104},{7773, 16578},{7774, 336779},{7775, -868276},{7776, -503736},{7777, 421161},{7778, 305054},{7779, -500813},{7780, -228601},
        {7781, 727532},{7782, 36809},{7783, 373396},{7784, 698504},{7785, 719634},{7786, 625435},{7787, -734924},{7788, 615000},{7789, -856063},{7790, 510836},{7791, 636959},{7792, -641026},{7793, -171127},{7794, 479413},{7795, 570913},{7796, -472592},{7797, 269740},{7798, 808974},{7799, -953994},{7800, -485824},
        {7801, 329946},{7802, -529301},{7803, 591638},{7804, -16149},{7805, 208191},{7806, 388533},{7807, 118318},{7808, 427010},{7809, 740637},{7810, -625402},{7811, -73216},{7812, -316723},{7813, -231069},{7814, -308864},{7815, 229974},{7816, -253687},{7817, -849816},{7818, -763670},{7819, 638283},{7820, 173599},
        {7821, -800038},{7822, 117535},{7823, -387123},{7824, -865775},{7825, 945300},{7826, 164973},{7827, 151238},{7828, -663788},{7829, 670307},{7830, -699522},{7831, -191072},{7832, 129041},{7833, 309485},{7834, 352969},{7835, 12549},{7836, 426362},{7837, -849077},{7838, -470893},{7839, 69595},{7840, -242346},
        {7841, -1017792},{7842, -530945},{7843, 509329},{7844, 770023},{7845, -622543},{7846, -889104},{7847, 774184},{7848, 356768},{7849, 546976},{7850, 588290},{7851, -307974},{7852, 363630},{7853, 724617},{7854, 417121},{7855, 12900},{7856, 753687},{7857, 262593},{7858, 752289},{7859, -81069},{7860, 271111},
        {7861, 239724},{7862, 224429},{7863, -254445},{7864, 625312},{7865, 571974},{7866, -73129},{7867, 457993},{7868, 113262},{7869, 308939},{7870, 905658},{7871, -798294},{7872, 35566},{7873, -574160},{7874, -516284},{7875, -556114},{7876, -258502},{7877, -570579},{7878, -274256},{7879, -57710},{7880, 19730},
        {7881, 304578},{7882, -901068},{7883, -371452},{7884, -624126},{7885, -629160},{7886, 237804},{7887, 102365},{7888, 851925},{7889, 212742},{7890, 765970},{7891, -705774},{7892, 504865},{7893, -84541},{7894, -529538},{7895, 689626},{7896, 254417},{7897, -961213},{7898, -777356},{7899, -827694},{7900, -169347},
        {7901, 28794},{7902, 605901},{7903, 417728},{7904, -85836},{7905, -631939},{7906, -224987},{7907, 782959},{7908, 331316},{7909, -205760},{7910, -640997},{7911, -13613},{7912, -160215},{7913, -106970},{7914, 506593},{7915, -694273},{7916, 73149},{7917, 380734},{7918, 199533},{7919, 463544},{7920, -327548},
        {7921, -246372},{7922, -447086},{7923, -730643},{7924, -11403},{7925, 201155},{7926, -742593},{7927, -197665},{7928, -183137},{7929, -759598},{7930, -609884},{7931, -604223},{7932, 480836},{7933, 599689},{7934, -207568},{7935, -614182},{7936, 19313},{7937, 601797},{7938, -143556},{7939, -264876},{7940, 292779},
        {7941, 600493},{7942, -340712},{7943, -714302},{7944, -483655},{7945, 553412},{7946, -73080},{7947, -630605},{7948, -735382},{7949, -73752},{7950, -383619},{7951, 654013},{7952, -33691},{7953, -936937},{7954, -400127},{7955, -398178},{7956, -629123},{7957, -633663},{7958, 726325},{7959, 379868},{7960, 510580},
        {7961, 1007768},{7962, 295700},{7963, -109153},{7964, 699116},{7965, 451279},{7966, -135646},{7967, 414176},{7968, -54381},{7969, 290040},{7970, 340431},{7971, 146503},{7972, -244325},{7973, -315756},{7974, -310258},{7975, 509781},{7976, -610359},{7977, -778541},{7978, -295580},{7979, -523989},{7980, 936013},
        {7981, 489638},{7982, -948096},{7983, 73075},{7984, -186363},{7985, -302964},{7986, -316328},{7987, -546344},{7988, -701730},{7989, -914734},{7990, 424936},{7991, 216603},{7992, 493211},{7993, 353718},{7994, -486165},{7995, -353319},{7996, -370109},{7997, -223031},{7998, 50257},{7999, 117925},{8000, 570693},
        {8001, -24055},{8002, 665225},{8003, 671591},{8004, -721308},{8005, -591778},{8006, 697096},{8007, 505529},{8008, -233626},{8009, -144273},{8010, -376836},{8011, -479041},{8012, -874192},{8013, 61313},{8014, -692302},{8015, 183769},{8016, 405689},{8017, -437988},{8018, -880746},{8019, -227021},{8020, 349381},
        {8021, -264678},{8022, 109427},{8023, -91958},{8024, -745911},{8025, -314105},{8026, -511356},{8027, -535367},{8028, -218294},{8029, -214344},{8030, -47415},{8031, 968059},{8032, -445899},{8033, 426328},{8034, 184715},{8035, 765867},{8036, 1038930},{8037, 567291},{8038, 793495},{8039, -451070},{8040, 12922},
        {8041, 110660},{8042, -316787},{8043, 743026},{8044, -257154},{8045, 229175},{8046, -193614},{8047, 687430},{8048, 216011},{8049, -495430},{8050, -431515},{8051, -741497},{8052, 887},{8053, 204822},{8054, 174353},{8055, 298167},{8056, -1021514},{8057, -751452},{8058, -491641},{8059, -642898},{8060, 222232},
        {8061, 504103},{8062, 494956},{8063, -282167},{8064, -320553},{8065, -752936},{8066, -720303},{8067, -624474},{8068, -469137},{8069, 787843},{8070, -108214},{8071, -663802},{8072, 654722},{8073, -200153},{8074, 581787},{8075, -473948},{8076, 91816},{8077, 457246},{8078, 363710},{8079, -568751},{8080, 542478},
        {8081, 522100},{8082, -582053},{8083, 240106},{8084, 524872},{8085, -97296},{8086, -668364},{8087, -569715},{8088, 927885},{8089, 550811},{8090, 982767},{8091, -591521},{8092, -675251},{8093, -87552},{8094, -79328},{8095, 630669},{8096, 299958},{8097, -922926},{8098, -515533},{8099, 2194},{8100, -548240},
        {8101, 531535},{8102, -298524},{8103, -83085},{8104, 57745},{8105, 265905},{8106, 140493},{8107, 49488},{8108, 255392},{8109, 620824},{8110, 10625},{8111, -378604},{8112, -538968},{8113, 559684},{8114, -361655},{8115, 768550},{8116, -964467},{8117, 488287},{8118, -522836},{8119, -275940},{8120, -490991},
        {8121, 744854},{8122, 352245},{8123, -447704},{8124, -567731},{8125, 573338},{8126, 774869},{8127, 542882},{8128, 415632},{8129, 262684},{8130, -460127},{8131, -545893},{8132, -69695},{8133, 442509},{8134, 634019},{8135, 90435},{8136, 320689},{8137, -728774},{8138, -362623},{8139, 473549},{8140, -249227},
        {8141, -294838},{8142, -857886},{8143, 445046},{8144, -492050},{8145, -114724},{8146, 37400},{8147, 199742},{8148, -623622},{8149, 638088},{8150, 160883},{8151, 755970},{8152, -442898},{8153, 614920},{8154, -356689},{8155, 778557},{8156, -636573},{8157, 416230},{8158, 246221},{8159, 48678},{8160, -732939},
        {8161, -407390},{8162, -798722},{8163, 478295},{8164, -658140},{8165, 696600},{8166, 600177},{8167, 606254},{8168, -289612},{8169, 27389},{8170, 1008308},{8171, -255908},{8172, 350915},{8173, 96682},{8174, 254613},{8175, 158640},{8176, -795109},{8177, -607498},{8178, 323645},{8179, -30318},{8180, -238728},
        {8181, 140355},{8182, 207774},{8183, 108222},{8184, -640781},{8185, 222762},{8186, -565016},{8187, -219025},{8188, -770016},{8189, 818709},{8190, -714004},{8191, 356651},{8192, -511775},{8193, -243521},{8194, -504766},{8195, 764148},{8196, -91643},{8197, 410953},{8198, 90515},{8199, -230466},{8200, 744600},
        {8201, 93534},{8202, 743006},{8203, -112992},{8204, 737915},{8205, -167579},{8206, -542750},{8207, 521388},{8208, 427451},{8209, 402272},{8210, 641696},{8211, -36724},{8212, 1008541},{8213, -477361},{8214, 3852},{8215, -654925},{8216, 227139},{8217, 214201},{8218, -564338},{8219, 242185},{8220, 669046},
        {8221, -868152},{8222, -265349},{8223, -438585},{8224, -302342},{8225, 226615},{8226, -916986},{8227, -7640},{8228, -809576},{8229, -518224},{8230, -57649},{8231, 348879},{8232, 688337},{8233, -153383},{8234, -343136},{8235, 560260},{8236, -96213},{8237, 684734},{8238, 610090},{8239, 359257},{8240, 629435},
        {8241, -691086},{8242, -224265},{8243, -185395},{8244, -286450},{8245, -628329},{8246, -327216},{8247, -255800},{8248, 387578},{8249, -631220},{8250, 90769},{8251, -761153},{8252, -158418},{8253, -955162},{8254, 17488},{8255, -604439},{8256, 160988},{8257, -38852},{8258, 403912},{8259, 832652},{8260, 598638},
        {8261, 703865},{8262, -521595},{8263, -237557},{8264, 595461},{8265, -89629},{8266, -38294},{8267, 721455},{8268, -24435},{8269, -123165},{8270, -640119},{8271, -645903},{8272, -661574},{8273, -294122},{8274, 534267},{8275, -791688},{8276, 492315},{8277, -94403},{8278, -162238},{8279, 90596},{8280, 143289},
        {8281, -654700},{8282, -716898},{8283, -63013},{8284, 340172},{8285, -115585},{8286, 442447},{8287, 18618},{8288, -114080},{8289, -276314},{8290, -730557},{8291, -574628},{8292, 694294},{8293, -403011},{8294, -117119},{8295, -570078},{8296, 775910},{8297, 151460},{8298, -378130},{8299, 250987},{8300, 72222},
        {8301, 107688},{8302, -329803},{8303, -532819},{8304, -566847},{8305, -194939},{8306, -599856},{8307, -424677},{8308, 57065},{8309, -349804},{8310, 616376},{8311, 254826},{8312, -750710},{8313, 651376},{8314, 157636},{8315, 518135},{8316, -727879},{8317, 331803},{8318, 30946},{8319, -485801},{8320, 559062},
        {8321, -42661},{8322, 456220},{8323, -662927},{8324, 718753},{8325, 192615},{8326, -419874},{8327, 429379},{8328, 710367},{8329, -33784},{8330, -788349},{8331, 752849},{8332, 283123},{8333, 412121},{8334, -546217},{8335, 34455},{8336, 118520},{8337, -320430},{8338, -823335},{8339, 738426},{8340, -147791},
        {8341, -568166},{8342, -544557},{8343, 923323},{8344, 504299},{8345, -66281},{8346, -469768},{8347, 735285},{8348, 87219},{8349, -664836},{8350, -254545},{8351, 689200},{8352, 769018},{8353, 783585},{8354, -45886},{8355, -684418},{8356, 702228},{8357, 979454},{8358, -559173},{8359, 46858},{8360, -337105},
        {8361, -700509},{8362, -40062},{8363, 146513},{8364, 678444},{8365, 612768},{8366, 416994},{8367, -402412},{8368, 13475},{8369, -476593},{8370, 23651},{8371, -351423},{8372, 743855},{8373, -1045581},{8374, -161386},{8375, 46942},{8376, -199389},{8377, 318664},{8378, 698705},{8379, 205982},{8380, -6782},
        {8381, -725763},{8382, -552484},{8383, -304800},{8384, 68778},{8385, 220482},{8386, -15705},{8387, 507818},{8388, -684262},{8389, 991790},{8390, -136320},{8391, -158428},{8392, 591581},{8393, -87471},{8394, -606067},{8395, 161094},{8396, -48809},{8397, -42397},{8398, 627206},{8399, 762418},{8400, -480404},
        {8401, -532020},{8402, -69636},{8403, -775340},{8404, -55446},{8405, 440587},{8406, 676832},{8407, -359772},{8408, -17017},{8409, 295722},{8410, -692605},{8411, 477714},{8412, -305771},{8413, -463578},{8414, 2441},{8415, -1016665},{8416, -708209},{8417, 64618},{8418, -734980},{8419, 273273},{8420, 621700},
        {8421, -390509},{8422, 402539},{8423, 157438},{8424, -551556},{8425, -340590},{8426, -258108},{8427, 628310},{8428, -445008},{8429, 404784},{8430, 74392},{8431, 165555},{8432, -871975},{8433, -632506},{8434, -285610},{8435, 617270},{8436, 663696},{8437, -685863},{8438, -101070},{8439, 248387},{8440, -289492},
        {8441, -121266},{8442, 473238},{8443, 728896},{8444, -702403},{8445, 565674},{8446, 666121},{8447, -701540},{8448, -166810},{8449, -481497},{8450, 13226},{8451, 412529},{8452, 716435},{8453, -247760},{8454, 185139},{8455, -615334},{8456, 684644},{8457, 254123},{8458, 1010714},{8459, -557529},{8460, 458825},
        {8461, -34977},{8462, 159187},{8463, -434836},{8464, -681665},{8465, 604880},{8466, 580735},{8467, 40157},{8468, 415360},{8469, 250673},{8470, 630155},{8471, -580980},{8472, 6475},{8473, -271254},{8474, 497124},{8475, -223867},{8476, 502833},{8477, 266507},{8478, 734092},{8479, 730068},{8480, 535307},
        {8481, 68967},{8482, -344880},{8483, 121675},{8484, -534550},{8485, 231223},{8486, 378483},{8487, -359073},{8488, -405968},{8489, 228482},{8490, 861503},{8491, -686432},{8492, -1030706},{8493, -471345},{8494, -818669},{8495, -146086},{8496, -304073},{8497, -414484},{8498, -386613},{8499, 530837},{8500, 112665},
        {8501, 428410},{8502, 673285},{8503, -246022},{8504, -422844},{8505, 35404},{8506, -186488},{8507, -511025},{8508, 713633},{8509, -87825},{8510, -693601},{8511, 97245},{8512, -50571},{8513, 477842},{8514, -541539},{8515, -633173},{8516, -44516},{8517, 528695},{8518, -720649},{8519, -702421},{8520, -595552},
        {8521, -230318},{8522, -238551},{8523, 16558},{8524, 403899},{8525, 951073},{8526, -167616},{8527, 270798},{8528, 457228},{8529, 257112},{8530, -1027158},{8531, 119353},{8532, -684683},{8533, 515194},{8534, -257867},{8535, -195462},{8536, -781581},{8537, 481769},{8538, -846975},{8539, -461682},{8540, -554371},
        {8541, 416973},{8542, 99858},{8543, -175588},{8544, -188545},{8545, -834536},{8546, -177334},{8547, 138017},{8548, -716025},{8549, 199171},{8550, 294972},{8551, 466867},{8552, 765736},{8553, -663933},{8554, -291992},{8555, -397299},{8556, 693653},{8557, -643983},{8558, -194667},{8559, -542092},{8560, -826082},
        {8561, 767126},{8562, -410968},{8563, -266334},{8564, -18011},{8565, 332381},{8566, 719698},{8567, 710673},{8568, -374926},{8569, -672756},{8570, -193096},{8571, 387289},{8572, 299197},{8573, -663902},{8574, -712168},{8575, 187794},{8576, 767340},{8577, 4832},{8578, -457702},{8579, 920013},{8580, 688185},
        {8581, -30792},{8582, 765415},{8583, -740595},{8584, 212736},{8585, -462399},{8586, 446544},{8587, 98587},{8588, 944645},{8589, -662395},{8590, 245640},{8591, -59726},{8592, 662898},{8593, -149900},{8594, 609083},{8595, -242108},{8596, 1034232},{8597, -88405},{8598, 732222},{8599, -404163},{8600, -588616},
        {8601, 536862},{8602, -771553},{8603, -905766},{8604, 522988},{8605, 124482},{8606, -118078},{8607, 446749},{8608, -3383},{8609, -341993},{8610, 8768},{8611, 289549},{8612, -613699},{8613, 53748},{8614, 666809},{8615, 260589},{8616, -405062},{8617, -82213},{8618, 319933},{8619, -599487},{8620, -329595},
        {8621, 233246},{8622, -727005},{8623, 747201},{8624, 656458},{8625, -511447},{8626, 578600},{8627, -479161},{8628, 426605},{8629, 168714},{8630, 121076},{8631, -409742},{8632, -102421},{8633, 354808},{8634, -169665},{8635, 321673},{8636, 377489},{8637, 640621},{8638, -49866},{8639, 202514},{8640, 157293},
        {8641, 507146},{8642, 560409},{8643, 312082},{8644, -51578},{8645, 61874},{8646, -665438},{8647, 666162},{8648, 280400},{8649, -538915},{8650, -620705},{8651, -76198},{8652, -631122},{8653, -492345},{8654, 363935},{8655, -474225},{8656, -93682},{8657, -364855},{8658, 466906},{8659, -495617},{8660, -3679},
        {8661, 673136},{8662, -281312},{8663, -510154},{8664, 160547},{8665, 346889},{8666, 141839},{8667, 314465},{8668, -88833},{8669, -129963},{8670, 495549},{8671, -165784},{8672, -168642},{8673, -451497},{8674, -387215},{8675, -701711},{8676, 229858},{8677, 243656},{8678, 192805},{8679, -727033},{8680, -671341},
        {8681, 766640},{8682, 634375},{8683, 273096},{8684, -559476},{8685, 114684},{8686, -1020195},{8687, 231852},{8688, 37102},{8689, -94171},{8690, 529390},{8691, 773950},{8692, 708230},{8693, -484455},{8694, -413942},{8695, 701550},{8696, -90250},{8697, -729361},{8698, 190656},{8699, 7303},{8700, -482794},
        {8701, -744161},{8702, -775718},{8703, -34027},{8704, 484127},{8705, 167194},{8706, 698351},{8707, 761591},{8708, -366597},{8709, 60814},{8710, 1026918},{8711, -703056},{8712, -13823},{8713, -585503},{8714, 647347},{8715, 757284},{8716, 441412},{8717, -326944},{8718, -93662},{8719, 616132},{8720, 340836},
        {8721, 119122},{8722, -69284},{8723, -566756},{8724, -314320},{8725, 295177},{8726, 203795},{8727, 497573},{8728, -1017046},{8729, -763330},{8730, 356062},{8731, 369543},{8732, -781657},{8733, -395366},{8734, -203445},{8735, -546576},{8736, -394354},{8737, 746835},{8738, -196163},{8739, -461072},{8740, -122727},
        {8741, -859436},{8742, -237473},{8743, -421174},{8744, -418971},{8745, -747080},{8746, -389899},{8747, 140178},{8748, -651348},{8749, 584038},{8750, 440476},{8751, 256366},{8752, 129955},{8753, -466431},{8754, -522440},{8755, -723809},{8756, 495922},{8757, 206044},{8758, 770154},{8759, -156340},{8760, 524938},
        {8761, -543246},{8762, 499141},{8763, -586992},{8764, 137072},{8765, -599100},{8766, -523007},{8767, -179677},{8768, 402993},{8769, -756164},{8770, 8211},{8771, -489564},{8772, 565425},{8773, -315354},{8774, -968522},{8775, 252570},{8776, -445803},{8777, -677318},{8778, -468570},{8779, -149657},{8780, -133033},
        {8781, 724533},{8782, 361328},{8783, -217864},{8784, 320346},{8785, 334390},{8786, 808460},{8787, -328418},{8788, -551243},{8789, 771518},{8790, 659595},{8791, -775477},{8792, 603547},{8793, 193914},{8794, 295829},{8795, 734688},{8796, 740286},{8797, -678940},{8798, 297787},{8799, -718122},{8800, 416221},
        {8801, -312341},{8802, 238883},{8803, -529090},{8804, -447781},{8805, -905164},{8806, 32649},{8807, 324412},{8808, 609137},{8809, 202660},{8810, 286454},{8811, 84265},{8812, 173586},{8813, -161929},{8814, 539225},{8815, -755998},{8816, 246689},{8817, -454360},{8818, 681435},{8819, 387731},{8820, -324394},
        {8821, -227899},{8822, 582040},{8823, 515225},{8824, -814289},{8825, 320381},{8826, -10448},{8827, -475982},{8828, -633795},{8829, 660787},{8830, -244726},{8831, 294244},{8832, 278274},{8833, 685459},{8834, 664740},{8835, -158103},{8836, 437164},{8837, 475705},{8838, 393747},{8839, -706305},{8840, 102471},
        {8841, 331951},{8842, -1018210},{8843, 650594},{8844, 43774},{8845, -659276},{8846, -535844},{8847, 155373},{8848, -593440},{8849, -231443},{8850, -931686},{8851, -139931},{8852, 158412},{8853, -566526},{8854, 342290},{8855, 408121},{8856, 700606},{8857, -469460},{8858, 773753},{8859, 162720},{8860, 290421},
        {8861, 784652},{8862, -388759},{8863, -37251},{8864, -172845},{8865, -284602},{8866, 734744},{8867, -515759},{8868, -36589},{8869, -562998},{8870, -739196},{8871, -573663},{8872, 17172},{8873, 606120},{8874, -22789},{8875, -553309},{8876, -359807},{8877, -637255},{8878, 29275},{8879, -135882},{8880, 230983},
        {8881, -271305},{8882, 573100},{8883, -655159},{8884, 681348},{8885, 220925},{8886, 554563},{8887, -84990},{8888, -525891},{8889, 890473},{8890, 288800},{8891, -109450},{8892, 566138},{8893, -607658},{8894, -49960},{8895, 167854},{8896, 297506},{8897, 484669},{8898, 440689},{8899, -533044},{8900, -583648},
        {8901, -248411},{8902, 576278},{8903, -427119},{8904, -118017},{8905, -118400},{8906, -771553},{8907, -321167},{8908, -783389},{8909, 508019},{8910, -114350},{8911, -606849},{8912, -204716},{8913, 125255},{8914, -489704},{8915, -435162},{8916, 780722},{8917, 601300},{8918, 560516},{8919, -399928},{8920, -464349},
        {8921, 126375},{8922, -341371},{8923, 755144},{8924, -471562},{8925, 41337},{8926, 676656},{8927, 447816},{8928, -593537},{8929, -34397},{8930, -638778},{8931, -512019},{8932, 213406},{8933, -310559},{8934, 217608},{8935, 260941},{8936, -314030},{8937, 22606},{8938, 882717},{8939, -198314},{8940, 294703},
        {8941, -534983},{8942, 596418},{8943, -571711},{8944, 370856},{8945, 502890},{8946, 721389},{8947, -262363},{8948, -515763},{8949, 896001},{8950, -456460},{8951, 331967},{8952, 387400},{8953, 699357},{8954, 260539},{8955, -129547},{8956, 373981},{8957, 431056},{8958, -958736},{8959, 104746},{8960, -382012},
        {8961, -120372},{8962, -767745},{8963, 422739},{8964, 863560},{8965, 109945},{8966, -238436},{8967, -1038808},{8968, 97822},{8969, 72507},{8970, 771418},{8971, -433851},{8972, -145378},{8973, 728737},{8974, 88016},{8975, -931799},{8976, 736549},{8977, 157881},{8978, -141518},{8979, -340840},{8980, -452449},
        {8981, 27355},{8982, -926372},{8983, -489652},{8984, 532504},{8985, 662601},{8986, 563667},{8987, -488446},{8988, 104504},{8989, -706017},{8990, -594373},{8991, -30142},{8992, -281166},{8993, 87158},{8994, 98650},{8995, 312601},{8996, 292971},{8997, 304301},{8998, -146865},{8999, -4243},{9000, -364615},
        {9001, -98740},{9002, -751493},{9003, -19823},{9004, -1030875},{9005, 986455},{9006, 259637},{9007, 120717},{9008, -902643},{9009, 586887},{9010, 257007},{9011, -27681},{9012, -387928},{9013, -566581},{9014, -198061},{9015, -213184},{9016, -506851},{9017, -245172},{9018, -118929},{9019, 1018085},{9020, 343911},
        {9021, -3015},{9022, -334038},{9023, 719821},{9024, 157922},{9025, -102825},{9026, -495192},{9027, -774128},{9028, -515312},{9029, 657940},{9030, 785373},{9031, 431711},{9032, 18584},{9033, 135604},{9034, 466262},{9035, 923870},{9036, 461079},{9037, 357398},{9038, 830841},{9039, 262032},{9040, -622919},
        {9041, 699163},{9042, 521300},{9043, -716164},{9044, 457330},{9045, 948007},{9046, 542845},{9047, 40952},{9048, -726536},{9049, 788918},{9050, 81010},{9051, -23010},{9052, -320716},{9053, 592353},{9054, 500676},{9055, -83526},{9056, -58114},{9057, -196958},{9058, -16657},{9059, 81057},{9060, -381242},
        {9061, 673663},{9062, 948930},{9063, 617696},{9064, -725790},{9065, 108018},{9066, -138807},{9067, -216149},{9068, -428695},{9069, -362493},{9070, 333829},{9071, 387027},{9072, 83474},{9073, 581035},{9074, 705033},{9075, 570756},{9076, 314693},{9077, -111467},{9078, -1003044},{9079, 146673},{9080, -157724},
        {9081, 94844},{9082, 414832},{9083, 303983},{9084, -1006675},{9085, 153898},{9086, -65221},{9087, 422817},{9088, 264991},{9089, -228471},{9090, -1040997},{9091, -31202},{9092, -772850},{9093, -73872},{9094, 936866},{9095, -801516},{9096, -692722},{9097, -19506},{9098, -178450},{9099, -462685},{9100, 249379},
        {9101, 631134},{9102, -65373},{9103, -151675},{9104, -333710},{9105, -276375},{9106, -576165},{9107, 188808},{9108, -521105},{9109, 225084},{9110, 441562},{9111, 52120},{9112, 939738},{9113, 843418},{9114, 475150},{9115, 512901},{9116, -361948},{9117, -233896},{9118, 325421},{9119, 683789},{9120, -496606},
        {9121, 944074},{9122, -489625},{9123, -196973},{9124, 42643},{9125, -815974},{9126, -777278},{9127, 355601},{9128, 1047729},{9129, 351148},{9130, 67306},{9131, -474711},{9132, 869347},{9133, -181793},{9134, -299865},{9135, -209054},{9136, 648846},{9137, 478713},{9138, 988786},{9139, 730812},{9140, -451896},
        {9141, 412080},{9142, 462840},{9143, 805821},{9144, -177495},{9145, 206716},{9146, -967313},{9147, 541948},{9148, 565473},{9149, -15890},{9150, 541481},{9151, 702831},{9152, 59048},{9153, 354192},{9154, 667468},{9155, -134753},{9156, 532232},{9157, 160037},{9158, 214086},{9159, 9323},{9160, 679705},
        {9161, -123972},{9162, 398285},{9163, -2269},{9164, -558733},{9165, 224515},{9166, 473301},{9167, 511558},{9168, -109505},{9169, 442586},{9170, -658083},{9171, -362333},{9172, -482315},{9173, -370117},{9174, -153563},{9175, 20180},{9176, -578541},{9177, -66157},{9178, -501591},{9179, 421659},{9180, -465200},
        {9181, 405401},{9182, -348024},{9183, 162288},{9184, 365446},{9185, -579977},{9186, 670391},{9187, 477158},{9188, 480133},{9189, 265864},{9190, 60064},{9191, 137504},{9192, 823691},{9193, 488783},{9194, -104441},{9195, 746537},{9196, 436117},{9197, -872154},{9198, -613430},{9199, -767665},{9200, 418807},
        {9201, 1001725},{9202, 258453},{9203, -656421},{9204, 283831},{9205, -731069},{9206, -906107},{9207, 100254},{9208, -463316},{9209, 589771},{9210, -156464},{9211, -807012},{9212, -280644},{9213, 641895},{9214, -764618},{9215, -614695},{9216, 516048},{9217, 593011},{9218, -56957},{9219, -598683},{9220, -717141},
        {9221, 451428},{9222, -317632},{9223, 252014},{9224, -763926},{9225, 498414},{9226, -941173},{9227, -60476},{9228, -236302},{9229, 867387},{9230, 959653},{9231, -386505},{9232, 157885},{9233, -349235},{9234, 755203},{9235, 183391},{9236, 125726},{9237, -636002},{9238, -696539},{9239, -357980},{9240, -1007946},
        {9241, -572688},{9242, -895615},{9243, 546090},{9244, 364829},{9245, 733671},{9246, -199613},{9247, 43713},{9248, 194689},{9249, -407927},{9250, 109985},{9251, 117935},{9252, 319211},{9253, -660745},{9254, -160957},{9255, 792736},{9256, -567936},{9257, -684208},{9258, 479653},{9259, 310079},{9260, 209935},
        {9261, 368931},{9262, -728762},{9263, 315776},{9264, -414683},{9265, -303881},{9266, -735828},{9267, 187229},{9268, -117660},{9269, 898610},{9270, 559772},{9271, 538539},{9272, -429266},{9273, -83237},{9274, 595902},{9275, -68506},{9276, -146987},{9277, 56746},{9278, -554603},{9279, -785680},{9280, 902017},
        {9281, 93789},{9282, -669954},{9283, 513768},{9284, 634322},{9285, -223082},{9286, 709273},{9287, -606001},{9288, 601294},{9289, -296838},{9290, 31104},{9291, -22485},{9292, -414461},{9293, -490574},{9294, -495967},{9295, -64139},{9296, -481809},{9297, -225273},{9298, 629210},{9299, 567165},{9300, 440997},
        {9301, 242600},{9302, 126964},{9303, -319354},{9304, 449699},{9305, -416999},{9306, -841683},{9307, 301529},{9308, -402837},{9309, -394323},{9310, 649442},{9311, 890739},{9312, 36453},{9313, 829565},{9314, 91245},{9315, -500945},{9316, -386872},{9317, 735701},{9318, 340586},{9319, -501693},{9320, -363154},
        {9321, 561637},{9322, 660176},{9323, 285143},{9324, -725996},{9325, 356890},{9326, 573590},{9327, -391313},{9328, -4223},{9329, -175544},{9330, -102392},{9331, 382932},{9332, 20874},{9333, 295018},{9334, -1045742},{9335, -619106},{9336, 82344},{9337, 584340},{9338, -432773},{9339, -338582},{9340, -627298},
        {9341, 718488},{9342, 515005},{9343, 627842},{9344, 533320},{9345, -457779},{9346, -209715},{9347, 514712},{9348, -757944},{9349, 690050},{9350, -574935},{9351, 536302},{9352, 474953},{9353, 926841},{9354, -147685},{9355, -262554},{9356, -517356},{9357, -1042552},{9358, -124499},{9359, 885747},{9360, -961005},
        {9361, 291199},{9362, 614496},{9363, -727891},{9364, 589290},{9365, 417529},{9366, -782962},{9367, 674716},{9368, 170785},{9369, 420810},{9370, -76828},{9371, -255812},{9372, -282775},{9373, 527978},{9374, 879764},{9375, 37646},{9376, 128865},{9377, 488284},{9378, -645737},{9379, -572989},{9380, -360135},
        {9381, -632710},{9382, -237566},{9383, -487752},{9384, 679851},{9385, -438517},{9386, 992490},{9387, -550980},{9388, 162080},{9389, 405677},{9390, -310751},{9391, -525631},{9392, 626207},{9393, 66227},{9394, 211666},{9395, -926487},{9396, -553866},{9397, 264358},{9398, -682506},{9399, -725244},{9400, -86965},
        {9401, 190733},{9402, 461283},{9403, 976442},{9404, -594323},{9405, -211406},{9406, 598064},{9407, 719696},{9408, 307261},{9409, -602857},{9410, 529211},{9411, 112317},{9412, -675130},{9413, -248990},{9414, 554640},{9415, -1044388},{9416, -878521},{9417, -500174},{9418, 229415},{9419, 375007},{9420, -311644},
        {9421, 629349},{9422, -195321},{9423, -144833},{9424, -142159},{9425, 75695},{9426, 102282},{9427, 679655},{9428, 335778},{9429, -554486},{9430, 410308},{9431, -77871},{9432, -788630},{9433, 540000},{9434, -670055},{9435, 901898},{9436, -393488},{9437, -79327},{9438, -720981},{9439, -174454},{9440, 13284},
        {9441, -106203},{9442, -393950},{9443, -319670},{9444, -703998},{9445, -671416},{9446, 67305},{9447, 346812},{9448, -587258},{9449, -662370},{9450, -572695},{9451, 602748},{9452, 791828},{9453, 325758},{9454, -1003118},{9455, -346777},{9456, 652316},{9457, 27372},{9458, -271897},{9459, -438045},{9460, -255453},
        {9461, 117367},{9462, -293768},{9463, 886479},{9464, -88962},{9465, 523062},{9466, -643497},{9467, 732221},{9468, -602510},{9469, -480588},{9470, 289057},{9471, 693400},{9472, 361681},{9473, -141575},{9474, 297815},{9475, 144742},{9476, -8582},{9477, 444558},{9478, -485355},{9479, -77762},{9480, -602529},
        {9481, 182218},{9482, 568680},{9483, -91583},{9484, 989562},{9485, 562380},{9486, 328684},{9487, 520473},{9488, -504623},{9489, 408572},{9490, 454285},{9491, 678607},{9492, 302528},{9493, -462722},{9494, 9741},{9495, 69636},{9496, -57535},{9497, 784368},{9498, -793266},{9499, -485732},{9500, 177548},
        {9501, 178261},{9502, -642322},{9503, 60159},{9504, -771116},{9505, -218959},{9506, -731581},{9507, 334842},{9508, 669143},{9509, -699089},{9510, -461561},{9511, -176426},{9512, 656445},{9513, 673729},{9514, 503889},{9515, -507032},{9516, 422147},{9517, 228167},{9518, 593645},{9519, 937265},{9520, 365932},
        {9521, 542227},{9522, 172407},{9523, 221838},{9524, 914771},{9525, 424402},{9526, -474528},{9527, -438631},{9528, -39420},{9529, -457126},{9530, -255218},{9531, -293019},{9532, 127397},{9533, 244742},{9534, 708355},{9535, -335170},{9536, 401163},{9537, 814349},{9538, -580356},{9539, 564138},{9540, 735756},
        {9541, -551743},{9542, -431788},{9543, 670380},{9544, -546389},{9545, -726054},{9546, 691119},{9547, -671374},{9548, -668655},{9549, 508756},{9550, 435044},{9551, 646713},{9552, -360856},{9553, 188962},{9554, -531417},{9555, -91824},{9556, 177845},{9557, -116152},{9558, 567911},{9559, -712986},{9560, 562238},
        {9561, 54860},{9562, 734580},{9563, -774081},{9564, -99341},{9565, -848931},{9566, -1006916},{9567, -172753},{9568, 352637},{9569, -32418},{9570, -650618},{9571, 220867},{9572, -643001},{9573, -126211},{9574, -509345},{9575, -286317},{9576, 525894},{9577, -82567},{9578, -3333},{9579, -251446},{9580, 52697},
        {9581, -741916},{9582, 738217},{9583, -224900},{9584, 1029863},{9585, 982931},{9586, -20448},{9587, 187828},{9588, 862425},{9589, 345367},{9590, -546036},{9591, 265410},{9592, -1042169},{9593, -167789},{9594, -91067},{9595, 162013},{9596, -382532},{9597, 298525},{9598, -645530},{9599, 638614},{9600, 26847},
        {9601, -819733},{9602, 738233},{9603, 62023},{9604, 401704},{9605, -54679},{9606, 753136},{9607, -281662},{9608, 654921},{9609, -487449},{9610, 470888},{9611, -454318},{9612, -379217},{9613, -702362},{9614, 771450},{9615, -296622},{9616, 42215},{9617, -215933},{9618, 380422},{9619, -342975},{9620, -256685},
        {9621, -363531},{9622, -825667},{9623, -503790},{9624, 163623},{9625, -1042838},{9626, -754763},{9627, -461267},{9628, -660647},{9629, -517840},{9630, 347749},{9631, 383447},{9632, -6533},{9633, 348475},{9634, 360006},{9635, 764428},{9636, 505936},{9637, 657944},{9638, -846019},{9639, 342722},{9640, 845251},
        {9641, -280090},{9642, -35165},{9643, 43948},{9644, -462217},{9645, -670071},{9646, 62814},{9647, -341779},{9648, 292947},{9649, 217650},{9650, -562110},{9651, 2362},{9652, -46189},{9653, -532087},{9654, -317219},{9655, 636223},{9656, -383166},{9657, 803418},{9658, -775793},{9659, -24413},{9660, 518276},
        {9661, -89092},{9662, 412332},{9663, 41732},{9664, 369127},{9665, -1005465},{9666, 3549},{9667, -61945},{9668, -291161},{9669, 864185},{9670, -90654},{9671, -666289},{9672, -686305},{9673, 709521},{9674, 516159},{9675, -431380},{9676, -255062},{9677, 573112},{9678, -185765},{9679, 531821},{9680, -1043209},
        {9681, -517430},{9682, 640777},{9683, 84388},{9684, -20456},{9685, -333070},{9686, -268441},{9687, 669976},{9688, 343391},{9689, -113636},{9690, 556924},{9691, 201301},{9692, 251886},{9693, -146062},{9694, -300145},{9695, 427868},{9696, 712967},{9697, 661055},{9698, -498548},{9699, 240244},{9700, 272843},
        {9701, -640682},{9702, -13943},{9703, 731857},{9704, 403024},{9705, -206237},{9706, 477820},{9707, 197190},{9708, 6550},{9709, 399027},{9710, -111323},{9711, 554666},{9712, 577952},{9713, -234232},{9714, 600045},{9715, 387126},{9716, -29672},{9717, 671053},{9718, -381782},{9719, 316215},{9720, -41959},
        {9721, -860029},{9722, 303545},{9723, 337518},{9724, -218698},{9725, 748064},{9726, 41448},{9727, 419616},{9728, -851893},{9729, 689825},{9730, -241782},{9731, -485627},{9732, 329742},{9733, -510569},{9734, 529744},{9735, 377947},{9736, 1023580},{9737, -175611},{9738, 315999},{9739, -735111},{9740, -98597},
        {9741, 40220},{9742, 176404},{9743, 132744},{9744, -657943},{9745, 284720},{9746, -253545},{9747, 247700},{9748, 52113},{9749, -426082},{9750, -197969},{9751, -414195},{9752, 954029},{9753, 153005},{9754, -614863},{9755, 225448},{9756, -423549},{9757, 75235},{9758, -337015},{9759, 206001},{9760, 92360},
        {9761, -67305},{9762, -246100},{9763, -287792},{9764, 177403},{9765, -598749},{9766, 37996},{9767, 224195},{9768, 264377},{9769, 475413},{9770, 552606},{9771, -559157},{9772, 5807},{9773, -224728},{9774, -649478},{9775, -559588},{9776, 392264},{9777, 742406},{9778, -469711},{9779, -537440},{9780, 347836},
        {9781, 456503},{9782, -50584},{9783, 414701},{9784, -20683},{9785, 460830},{9786, -207191},{9787, -476257},{9788, 584914},{9789, 529412},{9790, 424869},{9791, -219429},{9792, -303111},{9793, 23742},{9794, -266995},{9795, -672864},{9796, 255503},{9797, 467550},{9798, 32745},{9799, -752040},{9800, -652787},
        {9801, -548055},{9802, -334225},{9803, -345475},{9804, 431102},{9805, 132649},{9806, 100617},{9807, 480549},{9808, 360261},{9809, 684637},{9810, 384230},{9811, -741139},{9812, 365914},{9813, 340548},{9814, 212250},{9815, -676643},{9816, 72255},{9817, -548196},{9818, 346346},{9819, 779387},{9820, -966505},
        {9821, -693724},{9822, -117245},{9823, -172483},{9824, 735712},{9825, 287166},{9826, -56128},{9827, 88238},{9828, 122756},{9829, -549232},{9830, -160690},{9831, 229001},{9832, 608900},{9833, -127973},{9834, -606552},{9835, -151830},{9836, 167244},{9837, -365167},{9838, -572672},{9839, -535330},{9840, 784448},
        {9841, -789033},{9842, -722419},{9843, -133447},{9844, -202647},{9845, -467261},{9846, -692929},{9847, 677528},{9848, 489965},{9849, 461396},{9850, -606077},{9851, 196943},{9852, -989105},{9853, 728716},{9854, -783344},{9855, -646923},{9856, -64021},{9857, 8030},{9858, -664252},{9859, 172919},{9860, -435866},
        {9861, -442862},{9862, -247489},{9863, 406508},{9864, 302984},{9865, 469173},{9866, -240248},{9867, -785741},{9868, -107965},{9869, 657567},{9870, 760861},{9871, 671388},{9872, 753224},{9873, -925920},{9874, -784556},{9875, 202123},{9876, 362542},{9877, 438107},{9878, 344127},{9879, -81343},{9880, -596374},
        {9881, -201066},{9882, 1024944},{9883, -584149},{9884, 742856},{9885, -468300},{9886, 285315},{9887, 878237},{9888, 13509},{9889, -413597},{9890, -489937},{9891, 129869},{9892, -207677},{9893, 256479},{9894, -379851},{9895, -221386},{9896, 442256},{9897, 692500},{9898, 583253},{9899, 663439},{9900, 596266},
        {9901, -916530},{9902, -424766},{9903, -231560},{9904, 707282},{9905, 718558},{9906, -633922},{9907, 737583},{9908, 454982},{9909, 232804},{9910, 815783},{9911, 541468},{9912, 46864},{9913, 14690},{9914, -88280},{9915, 78629},{9916, -611342},{9917, -462553},{9918, -537808},{9919, -230187},{9920, -388725},
        {9921, -626815},{9922, -591851},{9923, 779854},{9924, 287715},{9925, 241716},{9926, 658714},{9927, -64077},{9928, 261716},{9929, 427844},{9930, 630762},{9931, 610270},{9932, 564517},{9933, -470703},{9934, -474380},{9935, 131449},{9936, 714682},{9937, -139053},{9938, -590653},{9939, -429644},{9940, -670900},
        {9941, 86227},{9942, -563925},{9943, 388131},{9944, 712947},{9945, -18747},{9946, -515987},{9947, -653265},{9948, -600727},{9949, -315011},{9950, 371343},{9951, 383915},{9952, -898230},{9953, 387782},{9954, 200886},{9955, -652802},{9956, 601493},{9957, 367338},{9958, 327812},{9959, 343889},{9960, -685151},
        {9961, 536789},{9962, -4498},{9963, 19019},{9964, 610698},{9965, -111225},{9966, -798649},{9967, 208121},{9968, -725383},{9969, 563098},{9970, 574257},{9971, 225143},{9972, -2723},{9973, -281792},{9974, 161294},{9975, 657800},{9976, 22308},{9977, 466158},{9978, -77917},{9979, -333033},{9980, 765509},
        {9981, 90817},{9982, 921927},{9983, 313885},{9984, -297356},{9985, 310814},{9986, -461858},{9987, -595435},{9988, 445833},{9989, 533920},{9990, -220005},{9991, -519265},{9992, 321472},{9993, 423903},{9994, -741091},{9995, 518869},{9996, -1036450},{9997, -5936},{9998, 335907},{9999, -180945},{10000, 720194},
         };
	int res = binary_search(data, 9989);

#ifdef TEST
    if(res == 533920){
        int ok = 1;
        memcpy(out_mem, &ok, sizeof(int));
        return;
    }
    int ok = 0;
    memcpy(out_mem, &ok, sizeof(int));
#endif
}

int binary_search(struct DATA data[], int x)
{
  int fvalue, mid, up, low ;

  low = 0;
  up = 9999;
  fvalue = -1 /* all data are positive */ ;
  while (low <= up) {
    mid = (low + up) >> 1;
    if ( data[mid].key == x ) {  /*  found  */
      up = low - 1;
      fvalue = data[mid].value;
    }
    else  /* not found */
      if ( data[mid].key > x ) 	{
	up = mid - 1;
      }
      else   {
          low = mid + 1;
      }

  }
  return fvalue;
}
